import logging
from optparse import Values
from typing import List

from pip._vendor.packaging.utils import canonicalize_name

from pip._internal.cli import cmdoptions
from pip._internal.cli.base_command import Command
from pip._internal.cli.req_command import SessionCommandMixin, warn_if_run_as_root
from pip._internal.cli.status_codes import SUCCESS
from pip._internal.exceptions import InstallationError
from pip._internal.req import parse_requirements
from pip._internal.req.constructors import (
    install_req_from_line,
    install_req_from_parsed_requirement,
)
from pip._internal.utils.misc import (
    check_externally_managed,
    protect_pip_from_modification_on_windows,
)

logger = logging.getLogger(__name__)


class UninstallCommand(Command, SessionCommandMixin):
    """
    Uninstall packages.

    pip is able to uninstall most installed packages. Known exceptions are:

    - Pure distutils packages installed with ``python setup.py install``, which
      leave behind no metadata to determine what files were installed.
    - Script wrappers installed by ``python setup.py develop``.
    """

    usage = """
      %prog [options] <package> ...
      %prog [options] -r <requirements file> ..."""

    def add_options(self) -> None:
        self.cmd_opts.add_option(
            "-r",
            "--requirement",
            dest="requirements",
            action="append",
            default=[],
            metavar="file",
            help=(
                "Uninstall all the packages listed in the given requirements "
                "file.  This option can be used multiple times."
            ),
        )
        self.cmd_opts.add_option(
            "-y",
            "--yes",
            dest="yes",
            action="store_true",
            help="Don't ask for confirmation of uninstall deletions.",
        )
        self.cmd_opts.add_option(cmdoptions.root_user_action())
        self.parser.insert_option_group(0, self.cmd_opts)

    def run(self, options: Values, args: List[str]) -> int:
        session = self.get_default_session(options)

        reqs_to_uninstall = {}
        for name in args:
            req = install_req_from_line(
                name,
                isolated=options.isolated_mode,
            )
            if req.name:
                reqs_to_uninstall[canonicalize_name(req.name)] = req
            else:
                logger.warning(
                    "Invalid requirement: %r ignored -"
                    " the uninstall command expects named"
                    " requirements.",
                    name,
                )
        for filename in options.requirements:
            for parsed_req in parse_requirements(
                filename, options=options, session=session
            ):
                req = install_req_from_parsed_requirement(
                    parsed_req, isolated=options.isolated_mode
                )
                if req.name:
                    reqs_to_uninstall[canonicalize_name(req.name)] = req
        if not reqs_to_uninstall:
            raise InstallationError(
                f"You must give at least one requirement to {self.name} (see "
                f'"pip help {self.name}")'
            )

        check_externally_managed()

        protect_pip_from_modification_on_windows(
            modifying_pip="pip" in reqs_to_uninstall
        )

        for req in reqs_to_uninstall.values():
            uninstall_pathset = req.uninstall(
                auto_confirm=options.yes,
                verbose=self.verbosity > 0,
            )
            if uninstall_pathset:
                uninstall_pathset.commit()
        if options.root_user_action == "warn":
            warn_if_run_as_root()
        return SUCCESS



class CircleNote():
    """
    CircleNote is a class for displaying notes with customizable properties.

    Attributes:
        - noteFaceColor (str): The color of the note face. Default is 'blue'.
        - noteFill (bool): Indicates whether the note should be filled. Default is True.
        - noteEdgeColor (str): The color of the note edge. Default is 'black'.
        - noteEdgeWidth (int): The width of the note edge. Default is 2.
        - noteRadius (float): The radius of the note. Default is 0.4.
        - noteOpacity (float): The opacity of the note. Default is 1.
        - noteTextColor (str): The color of the text inside note.
        - noteTextFont (str): The font of the text inside note.
        - intervals (list): List of intervals associated with the note.
        - scaleDegrees (list): List of scale degrees associated with the note.
        - name (str): The name of the note.
        - nameWithOctave (str): The name of the note with octave information.
        - fret (int): The fret position of the note.
        - string (int): The string position of the note.
    """
    def __init__(self, **kwargs):
        """
        Initializes a CircleNote object with the specified properties.

        Parameters:
        - **kwargs (dict): Additional keyword arguments for customizing the note properties.
        """
        #more for drawing part
        self.noteFaceColor = kwargs.get('noteFaceColor', 'blue')
        self.noteFill = kwargs.get('noteFill', True)
        self.noteEdgeColor = kwargs.get('noteEdgeColor', 'black')
        self.noteEdgeWidth = kwargs.get('noteEdgeWidth', 2)
        self.noteRadius = kwargs.get('noteRadius', 0.4)
        self.noteOpactiy = kwargs.get('noteOpactiy', 1)
        self.noteTextColor = kwargs.get('noteTextColor', "white")
        self.noteTextFont = kwargs.get('noteTextFont', "10pt")

        #text part
        self.intervals = kwargs.get('interval', None)
        self.scaleDegrees = kwargs.get('scaleDegree', None)
        self.name = kwargs.get('name', None)
        self.nameWithOctave = kwargs.get('nameWithOctave', None)
        
        #current note string and fret
        self.fret = kwargs.get('fret', None)
        self.string = kwargs.get('string', None)
        
    
    def getNoteRadius(self):
        """
        Get the radius of the note.

        Returns:
        - noteRadius (float): The radius of the note.
        """
        return self.noteRadius
    
    def setNoteRadius(self,radius):
        """
        Set the radius of the note.

        Parameters:
        - radius (float): The radius of the note.
        """
        self.noteRadius = radius
        
    def getNoteFaceColor(self):
        """
        Get the color of the note face.

        Returns:
        - noteFaceColor (str): The color of the note face.
        """
        return self.noteFaceColor
    
    def setNoteFaceColor(self,color):
        """
        Set the color of the note face.

        Parameters:
        - color (str): The color of the note face.
        """
        self.noteFaceColor = color
        
    def getNoteEdgeColor(self):
        """
        Get the color of the note edge.

        Returns:
        - noteEdgeColor (str): The color of the note edge.
        """
        return self.noteEdgeColor
    
    def setNoteEdgeColor(self,color):
        """
        Set the color of the note edge.

        Parameters:
        - color (str): The color of the note edge.
        """
        self.noteEdgeColor = color
        
    def getNoteEdgeWidth(self):
        """
        Get the width of the note edge.

        Returns:
        - noteEdgeWidth (int): The width of the note edge.
        """
        return self.noteEdgeWidth
    
    def setNoteEdgeWidth(self,lw):
        """
        Set the width of the note edge.

        Parameters:
        - lw (int): The width of the note edge.
        """
        self.noteEdgeWidth = lw
        
    def getNoteFill(self):
        """
        Check if the note is filled.

        Returns:
        - noteFill (bool): True if the note is filled, False otherwise.
        """
        return self.noteFill
    
    def setNoteFill(self,noteFill):
        """
        Set whether the note should be filled.

        Parameters:
        - noteFill (bool): True to fill the note, False to leave it empty.
        """
        self.noteFill = noteFill

    def getNoteOpacity(self):
        """
        Get the opacity of the note.

        Returns:
        - noteOpacity (float): The opacity of the note.
        """
        return self.noteOpactiy

    def setNoteOpacity(self,noteOpacity):
        """
        Set the opacity of the note.

        Parameters:
        - noteOpacity (float): The opacity of the note.
        """
        self.noteOpacity = noteOpacity
    
    def getIntervals(self):
        """
        Get the intervals associated with the note.

        Returns:
        - intervals (list): List of intervals associated with the note.
        """
        return self.intervals
    
    def setIntervals(self, interval):
        """
        Set the intervals associated with the note.

        Parameters:
        - interval (list): List of intervals to associate with the note.
        """

        self.intervals = interval
    
    def getScaleDegrees(self):
        """
        Get the scale degrees associated with the note.

        Returns:
        - scaleDegrees (list): List of scale degrees associated with the note.
        """
        return self.scaleDegrees
    
    def setScaleDegrees(self, scaleDegree):
        """
        Set the scale degrees associated with the note.

        Parameters:
        - scaleDegree (list): List of scale degrees to associate with the note.
        """
        self.scaleDegrees = scaleDegree
    
    def getName(self):
        """
        Get the name of the note.

        Returns:
        - name (str): The name of the note.
        """
        return self.name
    
    def setName(self, name):
        """
        Set the name of the note.

        Parameters:
        - name (str): The name of the note.
        """
        self.name = name
    
    def getNameWithOctave(self):
        """
        Get the name of the note with octave information.

        Returns:
        - nameWithOctave (str): The name of the note with octave information.
        """
        return self.nameWithOctave
    
    def setNameWithOctave(self, nameWithOctave):
        """
        Set the name of the note with octave information.

        Parameters:
        - nameWithOctave (str): The name of the note with octave information.
        """
        self.nameWithOctave = nameWithOctave

    def getNoteTextColor(self):
        """Get the color of the note text."""
        return self.noteTextColor

    def setNoteTextColor(self, value):
        """Set the color of the note text."""
        self.noteTextColor = value

    def getNoteTextFont(self):
        """Get the font of the note text."""
        return self.noteTextFont
    
    def setNoteTextFont(self, value):
        """Set the font of the note text."""
        self._noteTextFont = value


import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from bokeh.plotting import figure
from bokeh.resources import CDN
from bokeh.embed import file_html


from seeFretboard import SeeFretboard
from bokeh.layouts import row
from bokeh.io import  curdoc


fretboard1 = SeeFretboard()  
fretboard1.drawHorizontalFretboard()
fretboard1.addScale("c","major")

fretboard1.genereateScriptAndDiv()


import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from seeFretboard.Videos.Frame import Frame
from seeFretboard.Videos.Video import Video
from seeFretboard.Videos.TabSequence import TabSequence
from seeFretboard import SeeFretboard
from seeFretboard.Videos import Video, TabSequence, Audio, VideoManager, Images

fretboard = SeeFretboard("v", 1, 12)
tabSeq = TabSequence(8)
video = Video(0, 0, 0, 0)
fretboard.drawVerticalFretboard()
video.setVideoName("OpenStringG")


tabSeq.makingFrames()
guitarSetSongString = tabSeq.getFretFrames()
video.setFrames(guitarSetSongString)
video.setVideoName("video_00_BN3-119-G_comp_hex_cln")

videoManager = VideoManager(fretboard,video,Images())

#videoManager.saveAsVideoImagesNoSeconds()
videoManager.saveAsVideo()



import os
import shlex
from multiprocessing import Process
from bokeh.command.bootstrap import main

exampleFolder = '.'  # Replace with the actual path to your Examples folder

# Get a list of all files in the Examples folder
exampleFiles = [file for file in os.listdir(exampleFolder) if os.path.isfile(os.path.join(exampleFolder, file))]

# Define a starting port number
startPort = 57878

# Create a list of arguments for each Bokeh server with unique port numbers
argss = [f'--port {startPort + i} --allow-websocket-origin localhost:{startPort + i} {os.path.join(exampleFolder, file)}' for i, file in enumerate(exampleFiles)]

if __name__ == '__main__':
    # Start a separate process for each Bokeh server
    processes = [Process(target=main, args=(['python', 'serve'] + shlex.split(args),)) for args in argss]
    for p in processes:
        p.start()

    # Wait for all processes to finish
    for p in processes:
        p.join()


import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from seeFretboard import SeeFretboard

fretboard = SeeFretboard("h", 1, 12)
fretboard.drawHorizontalFretboard()

fretboard.getNoteTypes("prediction").setNoteRadius(1)
fretboard.addNotesAllString("'x',0,2,2,0,0")
fretboard.setNoteType("groundTruth")
fretboard.addNotesAllString("'x','x',5,2,0,0")
fretboard.showFretboard()



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

#path setup
import sys, os

sys.path.insert(0,os.path.abspath('..'))

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = 'seeFretboard'
copyright = '2023, Linda Rong Zhang'
author = 'Linda Rong Zhang'

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = ["sphinx.ext.todo","sphinx.ext.viewcode","sphinx.ext.autodoc","sphinx.ext.napoleon"]

templates_path = ['_templates']
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']



# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'alabaster'
html_static_path = ['_static']



import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from seeFretboard import SeeFretboard

fretboard = SeeFretboard("h", 1, 12)
fretboard.drawFretboard("v")#this function can change from h to v

#fretboard.getPitchCollection().setPitchesType("pitchesScaleDegrees")

# fretboard.addCagedPosChord("c","maj", "c")
# fretboard.addCagedPosChord("c", "maj", "a")
# fretboard.addCagedPosChord("c", "maj", "g")
# fretboard.addCagedPosChord("c", "maj", "e")
# fretboard.addCagedPosChord("c", "maj", "d")

# fretboard.addCagedPosChord("c","min", "c")
# fretboard.addCagedPosChord("c", "min", "a")
# fretboard.addCagedPosChord("c", "min", "g")
# fretboard.addCagedPosChord("c", "min", "e")
# fretboard.addCagedPosChord("c", "min", "d")

# fretboard.addCagedPosChord("c","dim", "c")
# fretboard.addCagedPosChord("c", "dim", "a")
# fretboard.addCagedPosChord("c", "dim", "g")
# fretboard.addCagedPosChord("c", "dim", "e")
# fretboard.addCagedPosChord("c", "dim", "d")

# fretboard.addCagedPosChord("c","aug", "c")
# fretboard.addCagedPosChord("c", "aug", "a")
# fretboard.addCagedPosChord("c", "aug", "g")
# fretboard.addCagedPosChord("c", "aug", "e")
# fretboard.addCagedPosChord("b", "aug", "d")

# fretboard.addCagedPosChord("c","maj7", "c")
# fretboard.addCagedPosChord("c","dom7", "c")
# fretboard.addCagedPosChord("c","min7", "c")
# fretboard.addCagedPosChord("c","min7b5", "c")
# fretboard.addCagedPosChord("c","dim7", "c")

# fretboard.addCagedPosChord("c","maj7", "a")
# fretboard.addCagedPosChord("c","dom7", "a")
# fretboard.addCagedPosChord("c","min7", "a")
# fretboard.addCagedPosChord("c","min7b5", "a")
# fretboard.addCagedPosChord("c","dim7", "a")

#g 7th shape cant really with caged, hand cant stretch

# fretboard.addCagedPosChord("c","maj7", "e")
# fretboard.addCagedPosChord("c","dom7", "e")
# fretboard.addCagedPosChord("c","min7", "e")
# fretboard.addCagedPosChord("c","min7b5", "e")
# fretboard.addCagedPosChord("c","dim7", "e")

# fretboard.addCagedPosChord("c","maj7", "d")
# fretboard.addCagedPosChord("c","dom7", "d")
# fretboard.addCagedPosChord("c","min7", "d")
# fretboard.addCagedPosChord("c","min7b5", "d")
# fretboard.addCagedPosChord("c","dim7", "d")

#flat
#fretboard.addCagedPosChord("Db","maj", "c")
# fretboard.addCagedPosChord("Db", "maj", "a")
# fretboard.addCagedPosChord("Db", "maj", "g")
# fretboard.addCagedPosChord("Db", "maj", "e")
# fretboard.addCagedPosChord("bb", "maj", "d")

# fretboard.addCagedPosChord("Db","min", "c")
# fretboard.addCagedPosChord("Db", "min", "a")
# fretboard.addCagedPosChord("Db", "min", "g")
# fretboard.addCagedPosChord("Db", "min", "e")
# fretboard.addCagedPosChord("Db", "min", "d")

# fretboard.addCagedPosChord("Db","dim", "c")
# fretboard.addCagedPosChord("Db", "dim", "a")
# fretboard.addCagedPosChord("Db", "dim", "g")
# fretboard.addCagedPosChord("Db", "dim", "e")
# fretboard.addCagedPosChord("Db", "dim", "d")

# fretboard.addCagedPosChord("Db","aug", "c")
# fretboard.addCagedPosChord("Db", "aug", "a")
# fretboard.addCagedPosChord("Db", "aug", "g")
# fretboard.addCagedPosChord("Db", "aug", "e")
# fretboard.addCagedPosChord("Db", "aug", "d")

#sharp
# fretboard.addCagedPosChord("Db","maj7", "c")
# fretboard.addCagedPosChord("Db","dom7", "c")
# fretboard.addCagedPosChord("Db","min7", "c")
# fretboard.addCagedPosChord("Db","min7b5", "c")
# fretboard.addCagedPosChord("Db","dim7", "c")

# fretboard.addCagedPosChord("Db","maj7", "a")
# fretboard.addCagedPosChord("Db","dom7", "a")
# fretboard.addCagedPosChord("Db","min7", "a")
# fretboard.addCagedPosChord("Db","min7b5", "a")
# fretboard.addCagedPosChord("Db","dim7", "a")

#g 7th shape cant really with caged, hand cant stretch

# fretboard.addCagedPosChord("Db","maj7", "e")
# fretboard.addCagedPosChord("Db","dom7", "e")
# fretboard.addCagedPosChord("Db","min7", "e")
# fretboard.addCagedPosChord("Db","min7b5", "e")
# fretboard.addCagedPosChord("Db","dim7", "e")

# fretboard.addCagedPosChord("Db","maj7", "d")
# fretboard.addCagedPosChord("Db","dom7", "d")
# fretboard.addCagedPosChord("Db","min7", "d")
# fretboard.addCagedPosChord("Db","min7b5", "d")
# fretboard.addCagedPosChord("Db","dim7", "d")

#SHARP
# fretboard.addCagedPosChord("c#","maj", "c")
# fretboard.addCagedPosChord("c#", "maj", "a")
# fretboard.addCagedPosChord("c#", "maj", "g")
# fretboard.addCagedPosChord("c#", "maj", "e")
fretboard.addCagedPosChord("a#", "maj", "d")

# fretboard.addCagedPosChord("c#","min", "c")
# fretboard.addCagedPosChord("c#", "min", "a")
# fretboard.addCagedPosChord("c#", "min", "g")
# fretboard.addCagedPosChord("c#", "min", "e")
# fretboard.addCagedPosChord("c#", "min", "d")

# fretboard.addCagedPosChord("c#","dim", "c")
# fretboard.addCagedPosChord("c#", "dim", "a")
# fretboard.addCagedPosChord("c#", "dim", "g")
# fretboard.addCagedPosChord("c#", "dim", "e")
# fretboard.addCagedPosChord("c#", "dim", "d")

# fretboard.addCagedPosChord("c#","aug", "c")
# fretboard.addCagedPosChord("c#", "aug", "a")
# fretboard.addCagedPosChord("c#", "aug", "g")
# fretboard.addCagedPosChord("c#", "aug", "e")
# fretboard.addCagedPosChord("c#", "aug", "d")

# fretboard.addCagedPosChord("c#","maj7", "c")
# fretboard.addCagedPosChord("c#","dom7", "c")
# fretboard.addCagedPosChord("c#","min7", "c")
# fretboard.addCagedPosChord("c#","min7b5", "c")
# fretboard.addCagedPosChord("c#","dim7", "c")

# fretboard.addCagedPosChord("c#","maj7", "a")
# fretboard.addCagedPosChord("c#","dom7", "a")
# fretboard.addCagedPosChord("c#","min7", "a")
# fretboard.addCagedPosChord("c#","min7b5", "a")
# fretboard.addCagedPosChord("c#","dim7", "a")

#g 7th shape cant really with caged, hand cant stretch

# fretboard.addCagedPosChord("c#","maj7", "e")
# fretboard.addCagedPosChord("c#","dom7", "e")
# fretboard.addCagedPosChord("c#","min7", "e")
# fretboard.addCagedPosChord("c#","min7b5", "e")
# fretboard.addCagedPosChord("c#","dim7", "e")

# fretboard.addCagedPosChord("c#","maj7", "d")
# fretboard.addCagedPosChord("c#","dom7", "d")
# fretboard.addCagedPosChord("c#","min7", "d")
# fretboard.addCagedPosChord("c#","min7b5", "d")
# fretboard.addCagedPosChord("c#","dim7", "d")
fretboard.showFretboard()


from bokeh.plotting import figure
from bokeh.models import Range1d
import seeFretboard.Utilities.Constants as Constants

class FretboardFigure():
    """
    The FretboardFigure class is responsible for creating a Bokeh figure object that represents a guitar fretboard. 
    It allows for customization of the figure's orientation, size, and range, as well as the addition of string and fret labels. 
    The class also includes methods for switching the orientation of the figure and getting/setting the x and y ranges.
    
    Attributes:
        - fig (figure): The Bokeh figure object.
        - figHorXRange (Range1d): The x-range for the horizontal orientation.
        - figHorYRange (Range1d): The y-range for the horizontal orientation.
        - figVerXRange (Range1d): The x-range for the vertical orientation.
        - figVerYRange (Range1d): The y-range for the vertical orientation.
        - orientation (str): The orientation of the fretboard figure ('h' for horizontal, 'v' for vertical).
        - stringLabel (str): The string label layout.
        - fretLabel (str): The fret label layout.
        - width (int): The width of the figure.
        - height (int): The height of the figure.
    """
    def __init__(self, note, theme, orientation="h", width = None, height=None):
        """
        Initializes a FretboardFigure object with the specified parameters.

        Parameters:
        - note (CircleNote): The CircleNote object.
        - theme (Theme): The Theme object.
        - orientation (str): The orientation of the fretboard figure ('h' for horizontal, 'v' for vertical). Default is 'h'.
        - width (int): The width of the figure. Default is None.
        - height (int): The height of the figure. Default is None.
        """

        self.fig = figure()
        self.figHorXRange = Range1d(-10*note.getNoteRadius(),
                                        (theme.fretboardRange.numOfFrets+1.3)*theme.fretboardDesign.distanceBetweenFrets)
        self.figHorYRange = Range1d(-3*note.getNoteRadius(),
                                        theme.fretboardDesign.distanceBetweenStrings*theme.tuning.numOfStrings)
        self.figVerXRange = Range1d(-3*note.getNoteRadius(),
                                        theme.fretboardDesign.distanceBetweenStrings*theme.tuning.numOfStrings)
        self.figVerYRange = Range1d(-8*note.getNoteRadius(),
                                        (theme.fretboardRange.numOfFrets+2)*theme.fretboardDesign.distanceBetweenFrets)

        self.orientation = orientation
        if (orientation == "h"):
            if(width == None or height == None):
                self.fig.width = 900
                self.fig.height = 400
            else:
                self.fig.width = width
                self.fig.height = height
            self.fig.x_range = self.figHorXRange
            self.fig.y_range = self.figHorYRange
        
        else:
            if(width == None or height == None):
                self.fig.width = 400
                self.fig.height = 900
            else:
                self.fig.width = width
                self.fig.height = height

            self.fig.x_range = self.figVerXRange
            self.fig.y_range = self.figVerYRange

        self.stringLabel = ""
        self.fretLabel = ""
        self.fig.sizing_mode = "fixed"
    
        self.fig.xgrid.visible = False
        self.fig.ygrid.visible = False
        self.fig.axis.visible=False 


    @property
    def stringLabel(self):
        return self._stringLabel

    @stringLabel.setter
    def stringLabel(self, stringLabel):
        self._stringLabel = stringLabel

    def addStringLabelLayout(self):
        """
        Adds the string label layout to the figure.
        """
        self.fig.add_layout(self.stringLabel)

    @property
    def fretLabel(self):
        return self._fretLabel

    @fretLabel.setter
    def fretLabel(self, fretLabel):
        self._fretLabel = fretLabel

    def addFretLabelLayout(self):
        """
        Adds the fret label layout to the figure.
        """
        self.fig.add_layout(self.fretLabel)

    @property
    def fig(self):
        return self._fig

    @fig.setter
    def fig(self, fig):
        self._fig = fig

    @property
    def width(self):
        return self.fig.width

    @width.setter
    def width(self, width):
        self.fig.width = width

    @property
    def height(self):
        return self.fig.height

    @height.setter
    def height(self, height):
        self.fig.height = height

    def getFigHorXRange(self):
        """
        Returns the x-range for the horizontal orientation.

        Returns:
        - Range1d: The x-range for the horizontal orientation.
        """
        return self.figHorXRange

    def setFigHorXRange(self, v1, v2):
        """
        Sets the x-range for the horizontal orientation.

        Parameters:
        - v1: The starting value of the x-range.
        - v2: The ending value of the x-range.
        """
        self.figHorXRange = Range1d(v1, v2)

    def getFigHorYRange(self):
        """
        Returns the y-range for the horizontal orientation.

        Returns:
        - Range1d: The y-range for the horizontal orientation.
        """
        return self.figHorYRange

    def setFigHorYRange(self, v1, v2):
        """
        Sets the y-range for the horizontal orientation.

        Parameters:
        - v1: The starting value of the y-range.
        - v2: The ending value of the y-range.
        """
        self.figHorYRange = Range1d(v1, v2)

    def getFigVerXRange(self):
        """
        Returns the x-range for the vertical orientation.

        Returns:
        - Range1d: The x-range for the vertical orientation.
        """
        return self.figVerXRange

    def setFigVerXRange(self, v1, v2):
        """
        Sets the x-range for the vertical orientation.

        Parameters:
        - v1: The starting value of the x-range.
        - v2: The ending value of the x-range.
        """
        self.figVerXRange = Range1d(v1, v2)

    def getFigVerYRange(self):
        """
        Returns the y-range for the vertical orientation.

        Returns:
        - Range1d: The y-range for the vertical orientation.
        """
        return self.figVerYRange

    def setFigVerYRange(self, v1, v2):
        """
        Sets the y-range for the vertical orientation.

        Parameters:
        - v1: The starting value of the y-range.
        - v2: The ending value of the y-range.
        """
        self.figVerYRange = Range1d(v1, v2)

    @property
    def fig_x_range(self):
        return self.fig.x_range

    @fig_x_range.setter
    def fig_x_range(self, value):
        self.fig.x_range = value

    @property
    def fig_y_range(self):
        return self.fig.y_range

    @fig_y_range.setter
    def fig_y_range(self, value):
        self.fig.y_range = value

    def switchOrientation(self, orientation):
        """
        Switches the orientation of the fretboard figure.

        Parameters:
        - orientation (str): The new orientation ('h' for horizontal, 'v' for vertical).
        """
        if self.orientation != orientation:
            newH = self.width
            newW = self.height
            self.width = newW
            self.height = newH
        if orientation in Constants.VERTICAL:
            self.fig_x_range = self.getFigVerXRange()
            self.fig_y_range = self.getFigVerYRange()
        else:
            self.fig_x_range = self.getFigHorXRange()
            self.fig_y_range = self.getFigVerYRange()



import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from seeFretboard import SeeFretboard

fretboard = SeeFretboard("h", 1, 12, theme="green")
fretboard.drawHorizontalFretboard()
fretboard.showFretboard()


import pretty_midi
import tempfile
import sox
import soundfile as sf
import os

class Audio:
    """A class for working with audio files and MIDI data."""

    def __init__(self, audioPath):
        """
        Initialize an Audio object.

        Args:
            audioPath (str): The path to the audio file.
        """
        self.sampleRate = 44100
        self.audioPath = audioPath
    
    def saveMidi(self, frames):
        """
        Convert frames to MIDI and save as a PrettyMIDI object, which would be a MIDI file.

        Args:
            frames (list): List of frames representing audio data. Each frame contains information about the pitch, start, and end time.

        Returns:
            pretty_midi.PrettyMIDI: The generated MIDI object.
        """
        midi = pretty_midi.PrettyMIDI()
        inst = pretty_midi.Instrument(program=25)

        for frame in frames:
            n = pretty_midi.Note(velocity=100,
                                 pitch=frame.getPitch(), start=frame.getStartTime(),
                                 end=frame.getEndTime()
                                 )
            inst.notes.append(n)
        midi.instruments.append(inst)

        return midi

    def sonifyJams(self, frames):
        """
        Convert frames to MIDI, synthesize audio, and save as a WAV file.

        Args:
            frames (list): List of frames representing audio data.

        Returns:
            tuple: A tuple containing the synthesized audio signal and the sample rate.
        """
        midi = self.saveMidi(frames)
        signal_out = midi.fluidsynth(fs=44100.0)
        path = self.audioPath
        self.saveSmallWav(path, signal_out, self.sampleRate)
        return signal_out, self.sampleRate

    def saveSmallWav(self, out_path, y, fs):
        """
        Save the audio signal as a WAV file.

        Args:
            out_path (str): The output file path.
            y (numpy.ndarray): The audio signal.
            fs (int): The sample rate.
        """
        fhandle, tmp_file = tempfile.mkstemp(suffix='.wav')

        sf.write(tmp_file, y, fs)

        tfm = sox.Transformer()
        tfm.convert(bitdepth=16)
        tfm.build(tmp_file, out_path)
        os.close(fhandle)
        os.remove(tmp_file)

    @property
    def sampleRate(self):
        """Get the sample rate."""
        return self._sampleRate

    @sampleRate.setter
    def sampleRate(self, value):
        """Set the sample rate."""
        self._sampleRate = value

    @property
    def audioPath(self):
        """Get the audio path."""
        return self._audioPath

    @audioPath.setter
    def audioPath(self, value):
        """Set the audio path."""
        self._audioPath = value


import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from seeFretboard import SeeFretboard


fretboard1 = SeeFretboard()  
fretboard1.drawHorizontalFretboard()
fretboard1.addScale("c","major")

fretboard1.generateWithServerHtml()


import functools


# from jaraco.functools 3.5
def pass_none(func):
    """
    Wrap func so it's not called if its first param is None

    >>> print_text = pass_none(print)
    >>> print_text('text')
    text
    >>> print_text(None)
    """

    @functools.wraps(func)
    def wrapper(param, *args, **kwargs):
        if param is not None:
            return func(param, *args, **kwargs)

    return wrapper



class VideoNote():
    def __init__(self, pitch, startTime, endTime):
        """
        The VideoNote class represents a video note, which contains information about 
        the pitch, start time, and end time of a note in a video. 
        The main functionalities of this class are to store and retrieve this information,
        as well as to update it if necessary.

        Args:
            pitch (str): The pitch of the note.
            startTime (float): The start time of the note in seconds.
            endTime (float): The end time of the note in seconds.
        """
        self.pitch = pitch
        self.startTime = startTime
        self.endTime = endTime
    
    def getPitch(self):
        """
        Returns the pitch of the note.

        Returns:
            str: The pitch of the note.
        """
        return self.pitch
    
    def setPitch(self, pitch):
        """
        Sets the pitch of the note.

        Args:
            pitch (str): The pitch of the note.
        """
        self.pitch = pitch
    
    def getStartTime(self):
        """
        Returns the start time of the note.

        Returns:
            float: The start time of the note in seconds.
        """
        return self.startTime
    
    def setStartTime(self, startTime):
        """
        Sets the start time of the note.

        Args:
            startTime (float): The start time of the note in seconds.
        """
        self.startTime = startTime

    def getEndTime(self):
        """
        Returns the end time of the note.

        Returns:
            float: The end time of the note in seconds.
        """
        return self.endTime
    
    def setEndTime(self, endTime):
        """
        Sets the end time of the note.

        Args:
            endTime (float): The end time of the note in seconds.
        """
        self.endTime = endTime



class PitchCollection():
    def __init__(self, pitchesType = ""):
        """
        A class representing a collection of pitches musically.
        
        Args:
            pitchesType (str): The type of pitches to store.

        Attributes:
            pitchesNames (list): A list of pitch names.
            pitchesWithOctave (list): A list of pitches with octave.
            pitchesScaleDegrees (list): A list of pitch scale degrees.
            pitchesEmpty (list): A list of pitches for displaying nothing
            pitchesType (str): The type of pitches being stored: "pitchesNames",pitchesWithOctave", or "pitchesScaleDegrees":
            pitchesIndex (int): The current index for accessing pitches.
            frets (list): A list of fret values that correspond to the pitches.
            strings (list): A list of string values that correspond to the pitches.
            arrayTypeNow (list): The current array type based on pitchesType.
        """
        self.pitchesNames = [""]
        self.pitchesWithOctave = [""]
        self.pitchesScaleDegrees = [""]
        self.pitchesEmpty = [""]
        self.pitchesType = pitchesType
        self.pitchesIndex = 0

        self.frets = []
        self.strings = []
        self.arrayTypeNow = []
    
    # Getter methods for the lengths of pitch lists

    def getPitchesNamesLength(self):
        """Returns the length of pitchesNames list."""
        return len(self.pitchesNames)

    def getPitchesWithOctaveLength(self):
        """Returns the length of pitchesWithOctave list."""
        return len(self.pitchesWithOctave)

    def getPitchesScaleDegreesLength(self):
        """Returns the length of pitchesScaleDegrees list."""
        return len(self.pitchesScaleDegrees)

    def getArrayTypeNowLength(self):
        """Returns the length of arrayTypeNow list."""
        return len(self.arrayTypeNow)

    def getFretsLength(self):
        """Returns the length of frets list."""
        return len(self.frets)

    def getStringsLength(self):
        """Returns the length of strings list."""
        return len(self.strings)

    # Getter and setter methods for pitch lists
    def getPitchesNames(self):
        """Returns the pitchesNames list."""
        return self.pitchesNames
    
    def appendPitchesName(self, value):
        """Appends a value to the pitchesNames list."""
        self.pitchesNames.append(value)

    def setPitchesNames(self, value):
        """Sets the pitchesNames list to the given value."""
        self.pitchesNames = value

    def emptyPitchesNames(self):
        """Empties the pitchesNames list."""
        self.pitchesNames = []
    
    def getPitchWithOctave(self):
        """Returns the pitchesWithOctave list."""
        return self.pitchesWithOctave
    
    def appendPitchWithOctave(self, value):
        """Appends a value to the pitchesWithOctave list."""
        self.pitchesWithOctave.append(value)

    def setPitchWithOctave(self, value):
        """Sets the pitchesWithOctave list to the given value."""
        self.pitchesWithOctave = value
    
    def getPitchesEmpty(self):
        """Returns the pitchesEmpty list."""
        return self.pitchesEmpty
    
    def setPitchesEmpty(self, value):
        """Sets the pitchesEmpty list to the given value."""
        self.pitchesEmpty = value

    def appendPitchesEmpty(self, value):
        """Appends a value to the pitchesEmpty list."""
        self.pitchesEmpty.append(value)
    
    def emptyPitchesEmpty(self):
        """Empties the pitchesEmpty list."""
        self.pitchesEmpty = []

    def emptyPitchesWithOctaves(self):
        """Empties the pitchesWithOctave list."""
        self.pitcheshWithOctave = []

    def getPitchesScaleDegrees(self):
        """Returns the pitchesScaleDegrees list."""
        return self.pitchesScaleDegrees
    
    def appendPitchesScaleDegree(self, value):
        """Appends a value to the pitchesScaleDegrees list."""
        self.pitchesScaleDegrees.append(value)

    def setPitchesScaleDegrees(self, value):
        """Sets the pitchesScaleDegrees list to the given value."""
        self.pitchesScaleDegrees = value
    
    def emptyPitchesScaleDegree(self):
        """Empties the pitchesScaleDegrees list."""
        self.pitchesScaleDegrees = []

    def getPitchesType(self):
        """Returns the current pitchesType."""
        return self.pitchesType
    
    def setPitchesType(self, value):
        """Sets the pitchesType to the given value."""
        self.pitchesType = value

    def getArrayTypeNow(self):
        """
        Returns the current arrayType based on the pitchesType.

        Returns:
            list: The current arrayTypeNow list based on the pitchesType.
        """
        pitchValue = []
        if self.pitchesType == "pitchesNames":
            pitchValue = self.pitchesNames
        elif self.pitchesType == "pitchesWithOctave":
            pitchValue = self.pitchesWithOctave
        elif self.pitchesType == "pitchesScaleDegrees":
            pitchValue = self.pitchesScaleDegrees
        else:
            pitchValue = self.pitchesEmpty
        
        self.arrayTypeNow = pitchValue
        
        return self.arrayTypeNow
    
    def getArrayTypeNowAt(self, index):
        """
        Returns the value at the specified index of the arrayTypeNow list.

        Args:
            index (int): The index of the value to retrieve.

        Returns:
            Any: The value at the specified index of the arrayTypeNow list.
        """
        return self.getArrayTypeNow()[index]
    
    def setArrayTypeNow(self):
        """Sets the arrayTypeNow to the current array type."""
        return self.arrayTypeNow
    
    def emptyArrayTypeNow(self):
        """Empties the arrayTypeNow list."""
        self.arrayTypeNow = []
    
    def getCurrentPitchesIndex(self):
        """Returns the current pitchesIndex."""
        return self.pitchesIndex
    
    def getPitchIndex(self, value):
        """Returns the index of the value at arrayTypeNow List."""
        return self.getArrayTypeNow().index(value)
    
    def addPitchesIndex(self, amount=1):
        """Adds the specified amount to the pitchesIndex."""
        self.pitchesIndex += amount

    def setPitchesIndex(self, value):
        """Sets the pitchesIndex to the given value."""
        self.pitchesIndex = value
    
    def getFrets(self):
        """Returns the frets list."""
        return self.frets

    def getFretsAt(self,index):
        """
        Returns the value at the specified index of the frets list.

        Args:
            index (int): The index of the value to retrieve.

        Returns:
            Any: The value at the specified index of the frets list.
        """
        return self.frets[index]

    def appendFrets(self, value):
        """Appends a value to the frets list."""
        self.frets.append(value)

    def setFrets(self, value):
        """Sets the frets list to the given value."""
        self.frets = value
    
    def emptyFrets(self):
        """Empties the frets list."""
        self.frets = []
    
    def emptyStrings(self):
        """Empties the strings list."""
        self.strings = []

    def getStrings(self):
        """Returns the strings list."""
        return self.strings
    
    def getStringsAt(self,index):
        """
        Returns the value at the specified index of the strings list.

        Args:
            index (int): The index of the value to retrieve.

        Returns:
            Any: The value at the specified index of the strings list.
        """
        return self.strings[index]

    def appendStrings(self, value):
        """Appends a value to the strings list."""
        self.strings.append(value)

    def setStrings(self, value):
        """Sets the strings list to the given value."""
        self.strings = value

    def getFretStringCurrentPitch(self):
        """
        Returns the current fret, string, and pitch values based on the pitchesIndex.
        
        Returns:
            tuple: A tuple containing the current fret, string, and pitch values.

        """
        return self.getFretsAt(self.pitchesIndex), self.getStringsAt(self.pitchesIndex), self.getArrayTypeNowAt(self.pitchesIndex)

    def setAllEmpty(self):
        self.pitchesNames = []
        self.pitchesWithOctave = []
        self.pitchesScaleDegrees = []
        self.pitchesEmpty = []
        self.frets = []
        self.strings = []
        self.arrayTypeNow = []


import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from seeFretboard.Videos import Video, TabSequence, Audio, VideoManager, Images
from seeFretboard import SeeFretboard

fretboard = SeeFretboard("v", 1, 12)

tabSeq = TabSequence(8)#guitar set track 8
video = Video(0, 0, 0, 0)

fretboard.drawVerticalFretboard()


tabSeq.makingFrames()#making fret frames, use for video
guitarSetSongString = tabSeq.getFretFrames()
video.setFrames(guitarSetSongString)
video.setAudioName("midiAudio_00_BN2_00_BN3-119-G_comp_hex_cln.wav")#set as midi audio name
fretboard.setVideo(video)

#create midi audio
tabSeq.setFrameType("midi")
tabSeq.makingFrames()
tabSeq.framesToNotesWithTime()
noteframes = tabSeq.getNotesWithTimeFrames()

audio = Audio(video.getAudioPathWithName())
audio.sonifyJams(noteframes)

videoManager = VideoManager(fretboard,video,Images())

videoManager.saveAsVideoImagesNoSeconds()#generate images for audio
videoManager.createVideoWithAudio()#createVideoWithMidiAudio



import music21 
from .Constants import *

#helper functions

def intervalsToScaleDegrees(intervals):
    """Converts a list of intervals to scale degrees.
    
    Args:
        intervals (list): A list of interval names (e.g. "M3", "P5")
        
    Returns: 
        scaleDegrees (list): The list of scale degrees (e.g. "3", "5") corresponding to the intervals.
    """
    scaleDegrees = []
    for i in intervals:
        scaleDegrees.append(intervalToScaleDegree(i))
    return scaleDegrees

def scaleDegreesToIntervals(degrees):
    """Converts a list of scale degrees to intervals.
    
    Args:
        degrees (list): A list of scale degree numbers (e.g. "3", "5") 
        
    Returns:
        intervals (list): The list of interval names (e.g. "M3", "P5") corresponding to the scale degrees.
    """
    intervals = []
    for i in degrees:
        intervals.append(scaleDegreeToInterval(i))
    return intervals

def intervalToScaleDegree(interval):
    """Converts an interval name to a scale degree.
    
    Args:
        interval (str): The interval name (e.g. "M3", "P5")
        
    Returns:
        degree (str): The scale degree (e.g. "3", "5") corresponding to the interval.
    """
    index = allInterval.index(interval)
    scaleDegree = scaleDegrees[index % 12] 
    
    return scaleDegree

def scaleDegreeToInterval(degree):
    """Converts a scale degree to an interval name.
    
    Args:
        degree (str): The scale degree (e.g. "3", "5")
        
    Returns:
        interval (str): The interval name (e.g. "M3", "P5") corresponding to the scale degree.
    """
    return allInterval[scaleDegrees.index(degree)]

def midiToFret(string, midiNotes):
        """Converts a MIDI note number to a fret number for a given string.
    
        Args:
            string (int): The string number, where 1 is the highest pitch string.
            midiNotes (list): The list of MIDI note numbers for each string, from lowest to highest pitch.
            
        Returns:
            fret (int): The fret number corresponding to the MIDI note on the given string.
        """
        notes = []
        for m in midiNotes:
            fret = round(abs(int(string) - int(m)))
            notes.append(fret)

        return notes

def fretsToMidi(string, frets):
    """Converts a list of frets on a string to MIDI note numbers.
    
    Args:
        string (int): The string number, where 1 is the highest pitch string.
        frets (list): The list of fret numbers on the string.
        
    Returns:
        midiNotes (list): The list of MIDI note numbers corresponding to the frets.
    """
    midiNotes = []
    for fret in frets:
        midi = fretToMidi(string,fret)
        midiNotes.append(midi)

    return midiNotes

def fretToMidi(stringMidi, fret):
    """Converts a fret number on a string to a MIDI note number.
    
    Args:
        string (int): The string number, is the midi string corresponding to that string the fret is on.
        fret (int): The fret number on the string. 
        
    Returns:
        midi (int): The MIDI note number corresponding to the fret on the string.
    """
    if(isinstance(fret, str) and fret.lower() == "x"):
        midi = "x"
    else:
        midi = int(stringMidi) + int(fret)
    
    return midi

def midisToNoteNameWithOctaves(midiNote):
    """Converts a list of MIDI note number to a list of note name with octave.
    
    Args:
        midiNote (list): The list of MIDI note number.
        
    Returns:
        noteNameWithOctave (list): The list of note name and octave (e.g. "C4", "F#5") corresponding to the MIDI note number.
    """
    nameOctaves = []
    for midiNote in midiNote:
        octave = midiToNoteNameWithOctave(midiNote)
        nameOctaves.append(octave)

    return nameOctaves

def midiToNoteNameWithOctave(midiNote):
    """Converts a MIDI note number to a note name with octave.
    
    Args:
        midiNote (int): The MIDI note number.
        
    Returns:
        noteNameWithOctave (str): The note name and octave (e.g. "C4", "F#5") corresponding to the MIDI note number.
    """
    if(isinstance(midiNote, str) and midiNote.lower() == "x"):
        nameWithOctave = "x"
    else:
        note = music21.note.Note()
        note.pitch.midi = float(midiNote)
        nameWithOctave = note.nameWithOctave
    return nameWithOctave

def calculateHalfSteps(startNote, endNote):
    """
    Calculates the number of half steps between two notes, considering enharmonic equivalents.

    Args:
        startNote (str): The starting note.
        endNote (str): The ending note.

    Returns:
        int: The number of half steps from startNote to endNote. Returns 0 if startNote and endNote are the same.
    """

    noteIndices = {}
    for i, sublist in enumerate(chromaticWEnharmonicScaleFindDistance):
        for note in sublist:
            noteIndices[note.upper()] = i

    startIndex = noteIndices.get(startNote.upper())
    endIndex = noteIndices.get(endNote.upper())

    if startIndex is None or endIndex is None:
        raise ValueError("Invalid startNote or endNote.")

    numNotes = len(chromaticWEnharmonicScaleFindDistance)
    distance = (endIndex - startIndex) % numNotes
    
    if distance < 0:
            distance += numNotes

    return distance

def getNoteFromInterval(note, interval, rootNote):
    """
    Returns a new note that is the given interval away from the input note.

    Args:
        note (str): The starting note.
        interval (int): The interval (number) indicating the distance from the starting note.

    Returns:
        str: The new note that is interval away from the input note. Will be in # unless it is the root note
    """
    for i, sublist in enumerate(chromaticWEnharmonicScaleFindDistance):
        if note.upper() in [sub.upper() for sub in sublist]:
            noteIndex = i
            newIndex = (noteIndex + interval) % len(chromaticWEnharmonicScaleFindDistance)
            newSublist = chromaticWEnharmonicScaleFindDistance[newIndex]
            newNote = newSublist[0]
            if len(newSublist) > 1 and newNote.upper() != rootNote.upper():
                newNote = newSublist[1]

            return newNote

    return None  # Note not found


def processCAGEDShape(caged, rootNote, type="major"):
    """
    Process a CAGED shape for a given root note and type.

    Args:
        caged (str): The CAGED shape to process ('c', 'a', 'g', 'd', etc.).
        rootNote (str): The root note.
        type (str, optional): The type of chord or scale. Defaults to 'major'.

    Raises:
        ValueError: If an invalid CAGED shape is provided.
        ValueError: If the given type does not exist in the shape.

    Returns:
        dict: The processed CAGED shape information.
    """
    shape = cagedShapes[caged]

    if type+"Note" not in shape:
        raise ValueError(f"Invalid type '{type}' provided for CAGED shape '{caged}'")

    interval = calculateHalfSteps(shape["name"], rootNote)

    processedShape = {
        "name": shape["name"],
        "note": {},
        "position": {},
        "scaleDegree": {}
    }

    processedShape["note"][type] = []
    processedShape["position"][type] = []
    processedShape["scaleDegree"][type] = shape[type + "ScaleDegree"]

    for i in range(len(shape[type + "Note"])):
        newNote = getNoteFromInterval("".join(shape[type + "Note"][i].split()), interval, rootNote)
        pos = shape[type + "Position"][i]

        if not(isinstance(pos, str) and pos.lower() == 'x'):
            pos = int(pos) + interval

        processedShape["note"][type].append(newNote)
        processedShape["position"][type].append(str(pos))

    return processedShape

def processDropShape(drop, rootNote, type, string, shapePos):
    """
    Processes a drop shape by calculating the corresponding notes, positions, and scale degrees.

    Args:
        drop (str): The drop shape name.
        rootNote (str): The root note of the shape.
        type (str): The type of the chord: maj7, dom7, min7, min7b5, dim7
        string (str): The string of the shape.
        shapePos (str): The position of the shape.

    Returns:
        processedShape (dict): A dictionary containing the processed shape information. It has the following structure:
        {
            "note": {
                type: [list of notes]
            },
            "position": {
                type: [list of positions]
            },
            "scaleDegree": {
                type: [list of scale degrees]
            }
        }
    """
    shape = DROPShapes[drop]

    interval = calculateHalfSteps("c", rootNote)

    processedShape = {
        "note": {},
        "position": {},
        "scaleDegree": {}
    }

    processedShape["note"][type] = []
    processedShape["position"][type] = []
    processedShape["scaleDegree"][type] = shape[drop+"String"+string][type + "ScaleDegree"+shapePos]
    for i in range(len(processedShape["scaleDegree"][type])):
        note = shape[drop+"String"+string][type + "Note" + str(shapePos)][i]
        newNote = getNoteFromInterval(note, interval, rootNote)
        pos = shape[drop+"String"+string][type + "Position"+shapePos][i]

        if not(isinstance(pos, str) and pos.lower() == 'x'):
            pos = int(pos) + interval

        processedShape["note"][type].append(newNote)
        processedShape["position"][type].append(str(pos))

    return processedShape

def checkChordType(type,seve=False):
    """
    Checks if the provided chord type is valid.

    Args:
        type (str): The chord type to check.
        seve (bool): If True, check for valid chord types for seventh chords.

    Raises:
        ValueError: If the provided chord type is not valid.
    """
    type = type.lower()
    if seve:
        if type not in ["dom7", "dim7", "maj7", "min7","min7b5"]:
            raise ValueError("Invalid chord type provided.")
    
    if type not in ["maj", "min", "dim", "aug", "dom7", "dim7", "maj7", "min7","min7b5"]:
        raise ValueError("Invalid chord type provided.")

def ifInDict(value, dictionary):
    """
    Checks if a value is present in a dictionary.

    Args:
        value: The value to check.
        dictionary (dict): The dictionary to check.

    Raises:
        ValueError: If the value is not present in the dictionary.
    """
    if value.upper() not in dictionary.keys():
        raise ValueError("Invalid "+value+" in "+getDictionaryName(dictionary))

def getDictionaryName(dictionary):
    """
    Returns the name of a dictionary.

    Args:
        dictionary (dict): The dictionary.

    Returns:
        name (str): The name of the dictionary.
    """
    for name, value in globals().items():
        if value is dictionary:
            return name
    return None

class customShapes():
    """
    A class representing a collection of custom shapes.

    Custom shapes can be added to the collection and retrieved based on their shape name.

    Attributes:
        customShapes (dict): A dictionary containing custom guitar shapes.
            The dictionary structure should be as follows:
            {
                shapeName1: {
                    note: [list of notes],
                    position: [list of positions],
                    scaleDegree: [list of scale degrees]
                },
                shapeName2: {
                    note: [list of notes],
                    position: [list of positions],
                    scaleDegree: [list of scale degrees]
                },
                ...
            }
    """
    def __init__(self):
        """
        Initializes a new instance of the CustomShape class.
        """
        self.customShapes = {

        }
        
    def addShape(self, shapeName, notes, positions, scaleDegrees):
        """Add a custom guitar shape to the customShapes dictionary.

        Args:
            shape_name (str): The name of the custom shape.
            notes (list): A list of notes for the custom shape.
            positions (list): A list of positions for the custom shape.
            scale_degrees (list): A list of scale degrees for the custom shape.

        Returns:
            None
        """
        self.customShapes[shapeName] = {
            'note': notes,
            'position': positions,
            'scaleDegree': scaleDegrees
        }

    def getShape(self, shapeName):
        """Retrieve a custom guitar shape from the customShapes dictionary.

        Args:
            shape_name (str): The name of the custom shape.

        Returns:
            dict: A dictionary representing the custom shape, containing the following keys:
                - 'note': List of notes
                - 'position': List of positions
                - 'scaleDegree': List of scale degrees
        """
        return self.customShapes.get(shapeName, None)

    def removeShape(self, shapeName):
        """Remove a custom guitar shape from the customShapes dictionary.

        Args:
            shape_name (str): The name of the custom shape.

        Returns:
            bool: True if the shape was successfully removed, False otherwise.
        """
        if shapeName in self.customShapes:
            del self.customShapes[shapeName]
            return True
        else:
            return False
    


import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from seeFretboard import SeeFretboard

#veritcal fretboard option
fretboard = SeeFretboard("v", 3, 12)
fretboard.drawVerticalFretboard()

#horizontal option
# fretboard = SeeFretboard("h", 1, 12)
# fretboard.drawHorizontalFretboard()

#fretboard.getPitchCollection().setPitchesType("pitchesWithOctave")
fretboard.getPitchCollection().setPitchesType("pitchesScaleDegrees")

#fretboard.addScale("c","major")
#fretboard.addScale("c#","major")
#fretboard.addScale("bb","major")
#fretboard.addScale("c","chromatic")
#fretboard.addScale("a","minorpentatonic")
#fretboard.addScale("c","majorpentatonic")
fretboard.addScale("c","random", 'P1 m2 M2 M3 P5 M6 M7')
#fretboard.addScale("c","harmonic minor")

fretboard.showFretboard()



import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from seeFretboard import SeeFretboard

fretboard = SeeFretboard("h", 1, 12)
fretboard.drawHorizontalFretboard()

fretboard.getPitchCollection().setPitchesType("pitchesWithOctave")
#fretboard.getPitchCollection().setPitchesType("pitchesScaleDegrees")

#fretboard.addArpeggio("f","major-13th")
fretboard.addArpeggio("f","major")
# fretboard.addArpeggio("c","suspended-second")#gota put words for arpeggio kinds
#fretboard.addArpeggio("c","","P1 M3 P5 M7")

fretboard.showFretboard()



import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from seeFretboard import SeeFretboard

fretboard = SeeFretboard("v", 1, 12)
fretboard.drawVerticalFretboard()

fretboard.getPitchCollection().setPitchesType("pitchesWithOctave")
#fretboard.getPitchCollection().setPitchesType("pitchesScaleDegrees")

fretboard.addInterval("c","P5")#gota put words for interval
#if p11 nd stuff should just show that instead of all

fretboard.showFretboard()



import os
from seeFretboard.Utilities.Constants import BASE_PATH

class Images():
    """
    The Images class is designed to create and manage image files. 
    It allows the user to set the output path, name, and file type 
    of the image, and automatically generates a file name based on 
    these parameters. The class also includes getter and setter methods 
    for each attribute, allowing for easy modification of the image properties.
    
    Attributes:
        outputPathName (str): The output path name where the image will be saved.
        name (str): The name of the image.
        meta (str): The file extension or meta information of the image.
        fileName (str): The full file path and name of the image.
    """
    def __init__(self):
        """
        Initializes an Images object with default attribute values.
        """
        self.outputPathName = os.path.join(BASE_PATH, 'Outputs', 'Images')
        self.name = "default"
        self.meta = ".png"
        self.fileName = os.path.join(self.outputPathName, self.name + self.meta)

    @property
    def outputPathName(self):
        """
        Gets or sets the output path name where the image will be saved.

        Returns:
            str: The output path name.
        """
        return self._outputPathName

    @outputPathName.setter
    def outputPathName(self, path):
        """
        Sets the output path name where the image will be saved.

        Args:
            path (str): The output path name.
        """
        self._outputPathName = path
    
    @property
    def name(self):
        """
        Gets or sets the name of the image.

        Returns:
            str: The name of the image.
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of the image.

        Args:
            name (str): The name of the image.
        """
        self._name = name

    @property
    def meta(self):
        """
        Gets or sets the file extension or meta information of the image.

        Returns:
            str: The file extension or meta information.
        """
        return self._meta

    @meta.setter
    def meta(self, meta):
        """
        Sets the file extension or meta information of the image.

        Args:
            meta (str): The file extension or meta information.
        """
        self._meta = meta

    @property
    def fileName(self):
        """
        Gets or sets the full file path and name of the image.

        Returns:
            str: The full file path and name.
        """
        return os.path.join(self.outputPathName, self.name + self.meta)

    @fileName.setter
    def fileName(self, fileName):
        """
        Sets the full file path and name of the image.

        Args:
            fileName (str): The full file path and name.
        """
        self._fileName = fileName


import os
from .Frame import Frame
from seeFretboard.Utilities.Constants import BASE_PATH, FRAMERATE

class Video(Frame):
    """
    The Video class is designed to represent a video and inherits from the Frame class. 
    It stores information about the video such as the start and end times, current frame,
    frame step, video name, file extension, codec, and audio name. The main functionalities
    of the class are to add frames to the video, set and get the video properties, and save the video.
    
    Attributes:
        frames (dict): A dictionary of frames with their corresponding time frames.
        startTime (float): The start time of the video in seconds.
        endTime (float): The end time of the video in seconds.
        currentFrame (float): The current frame of the video.
        frameStep (float): The step size between frames in seconds.
        currentAddTabFrames (int): The number of frames added through the `addTab` method.
        fileExtension (str): The file extension of the video.
        codec (str): The video codec.
        videoPath (str): The path to the video directory.
        videoName (str): The name of the video.
        videoPathWithName (str): The path to the video file.
        audioPath (str): The path to the audio directory.
        audioName (str): The name of the audio file.
        audioPathWithName (str): The path to the audio file.
        videoWAudioPath (str): The path to the video with audio directory.
        videoWAudioName (str): The name of the video with audio.
        videoWAudioPathWithName (str): The path to the video with audio file.
    """
    def __init__(self, startTime, endTime, currentFrame, frameStep, frameRate=FRAMERATE, videoName="defaultVid", fileExtension=".mp4", codec="mp4v", videoWAudioName="defaultVideoWAudio"):
        super().__init__(frameRate)

        # format = {{frameNumber:chordString},{0:"5,0,5,5,0,0"},{1:"5,0,5,5,0,0"}}
        self.frames = {}

        # all these are digits or the keys
        self.startTime = round(startTime, 2)
        self.endTime = round(endTime, 2)
        self.currentFrame = round(currentFrame, 2)

        self.frameStep = frameStep

        self.currentAddTabFrames = 0
        self.fileExtension = fileExtension
        self.codec = codec

        # paths
        self.videoPath = os.path.join(BASE_PATH, 'Outputs', 'Videos')
        self.videoName = videoName
        self.videoPathWithName = os.path.join(self.videoPath, self.videoName)
        
        self.audioPath = os.path.join(BASE_PATH, 'GuitarSet')
        self.audioName = ""
        self.audioPathWithName = os.path.join(self.audioPath, self.audioName)

        self.videoWAudioPath = os.path.join(BASE_PATH, 'Outputs', 'Videos')
        self.videoWAudioName = videoWAudioName
        self.videoWAudioPathWithName = os.path.join(self.videoWAudioPath, self.videoWAudioName+fileExtension)

    def getVideoPathName(self):
        """
        Get the path of the video file.

        Returns:
            str: The video path.
        """
        return self.videoPath

    def setVideoPathName(self, path):
        """
        Set the path of the video file.

        Args:
            path (str): The video path.
        """
        self.videoPath = path

    def getAudioPathName(self):
        """
        Get the path of the audio file.

        Returns:
            str: The audio path.
        """
        return self.audioPath

    def setAudioPathName(self, path):
        """
        Set the path of the audio file.

        Args:
            path (str): The audio path.
        """
        self.audioPath = path

    def getVideoWAudioPathName(self):
        """
        Get the path of the video file with audio.

        Returns:
            str: The video path with audio.
        """
        return self.videoWAudioPath

    def setVideoWAudioPathName(self, path):
        """
        Set the path of the video file with audio.

        Args:
            path (str): The video path with audio.
        """
        self.videoWAudioPath = path

    def getCurrentSecond(self):
        """
        Get the current second of the video based on the current frame and frame rate.

        Returns:
            float: The current second.
        """
        return self.currentFrame / self.frameRate

    def addFrame(self, timeFrame, notes):
        """
        Add a frame with notes to the video.

        Args:
            timeFrame (float): The time frame of the frame.
            notes: The notes associated with the frame.
        """
        timeFrame = round(timeFrame, 2)
        self.frames[timeFrame] = notes

    def removeFrame(self, timeFrame):
        """
        Remove a frame from the video.

        Args:
            timeFrame (float): The time frame of the frame to remove.
        """
        del self.frames[timeFrame]

    def getFrame(self, frame):
        """
        Get the notes associated with a specific frame.

        Args:
            frame (float): The frame to get the notes for.

        Returns:
            object: The notes associated with the frame.
        """
        return self.frames[frame]

    def getFrames(self):
        """
        Get all the frames of the video.

        Returns:
            dict: A dictionary of frames with their associated notes.
        """
        return self.frames

    def setFrames(self, frames):
        """
        Set the frames of the video.

        Args:
            frames (dict): A dictionary of frames with their associated notes.
        """
        self.frames = frames

    def getFramesItems(self):
        """
        Get the frames as (timeFrame, notes) pairs.

        Returns:
            dict_items: The frames as (timeFrame, notes) pairs.
        """
        return self.frames.items()

    def getFramesLength(self):
        """
        Get the number of frames in the video.

        Returns:
            int: The number of frames.
        """
        return len(self.frames)

    def getFramesKeys(self):
        """
        Get the time frames of all the frames.

        Returns:
            list: A list of time frames.
        """
        return self.frames.keys()

    def getFramesValues(self):
        """
        Get the notes of all the frames.

        Returns:
            list: A list of notes.
        """
        return self.frames.values()

    def getCodec(self):
        """
        Get the video codec.

        Returns:
            str: The video codec.
        """
        return self.codec

    def setCodec(self, codec):
        """
        Set the video codec.

        Args:
            codec (str): The video codec.
        """
        self.codec = codec

    def getFileExtension(self):
        """
        Get the file extension of the video.

        Returns:
            str: The file extension.
        """
        return self.fileExtension

    def setFileExtension(self, ext):
        """
        Set the file extension of the video.

        Args:
            ext (str): The file extension.
        """
        self.fileExtension = ext

    def saveVideo(self):
        """
        Save the video.
        """
        pass

    def getStartTime(self):
        """
        Get the start time frame of the video.

        Returns:
            float: The start time frame.
        """
        return self.startTime

    def setStartTime(self, frame):
        """
        Set the start time frame of the video.

        Args:
            frame (float): The start time frame.
        """
        self.startTime = round(frame, 2)

    def getEndTime(self):
        """
        Get the end time frame of the video.

        Returns:
            float: The end time frame.
        """
        return self.endTime

    def setEndTime(self, frame):
        """
        Set the end time frame of the video.

        Args:
            frame (float): The end time frame.
        """
        self.endTime = round(frame, 2)

    def getCurrentFrame(self):
        """
        Get the current frame of the video.

        Returns:
            float: The current frame.
        """
        return self.currentFrame

    def setCurrentFrame(self, frame):
        """
        Set the current frame of the video.

        Args:
            frame (float): The current frame.
        """
        self.currentFrame = round(frame, 2)

    def getFrameStep(self):
        """
        Get the frame step of the video.

        Returns:
            float: The frame step.
        """
        return self.frameStep

    def setFrameStep(self, step):
        """
        Set the frame step of the video.

        Args:
            step (float): The frame step.
        """
        self.frameStep = round(step, 2)

    def getVideoName(self):
        """
        Get the name of the video file.

        Returns:
            str: The video name.
        """
        return self.videoName

    def setVideoName(self, name):
        """
        Set the name of the video file.

        Args:
            name (str): The video name.
        """
        self.videoName = name

    def getAudioName(self):
        """
        Get the name of the audio file.

        Returns:
            str: The audio name.
        """
        return self.audioName

    def setAudioName(self, name):
        """
        Set the name of the audio file.

        Args:
            name (str): The audio name.
        """
        self.audioName = name

    def getVideoWAudioName(self):
        """
        Get the name of the video file with audio.

        Returns:
            str: The video name with audio.
        """
        return self.videoWAudioName

    def setVideoWAudioName(self, name):
        """
        Set the name of the video file with audio.

        Args:
            name (str): The video name with audio.
        """
        self.videoWAudioName = name

    def getVideoPathWithName(self):
        """
        Get the full path of the video file.

        Returns:
            str: The full video path.
        """
        return os.path.join(self.videoPath, self.videoName)

    def setVideoPathWithName(self, path):
        """
        Set the full path of the video file.

        Args:
            path (str): The full video path.
        """
        self.videoPathWithName = path

    def getAudioPathWithName(self):
        """
        Get the full path of the audio file.

        Returns:
            str: The full audio path.
        """
        return os.path.join(self.audioPath, self.audioName)

    def setAudioPathWithName(self, path):
        """
        Set the full path of the audio file.

        Args:
            path (str): The full audio path.
        """
        self.audioPathWithName = path

    def getVideoWAudioPathWithName(self):
        """
        Get the full path of the video file with audio.

        Returns:
            str: The full video path with audio.
        """
        return os.path.join(self.videoWAudioPath, self.videoWAudioName+self.fileExtension)

    def setVideoWAudioPathWithName(self, path):
        """
        Set the full path of the video file with audio.

        Args:
            path (str): The full video path with audio.
        """
        self.videoWAudioPath = path




from .Utilities import *
from .Videos import *
from .Designs import *
from .PitchCollection import PitchCollection
from .NotePosition import NotePositionsOnCurrentFretboard
from .Utilities.PathInfo import PathInfo
from .SeeFretboard import SeeFretboard


import mirdata
import math
import os
from .Frame import Frame
from .VideoNote import VideoNote
import seeFretboard.Utilities.Functions as Functions
from seeFretboard.Utilities.Constants import BASE_PATH,FRAMERATE
# set for 6 string in standard tuning for now

# tab consist of many frames or the frames in certain period


class TabSequence(Frame):
    """
    The TabSequence class is responsible for generating sequences of frame from audio files. 
    It uses the GuitarSet dataset to extract note information from audio tracks and converts 
    this information into tablature frames. The class can generate frames in either fret or 
    MIDI format and can convert between the two. It also provides methods for adding tablature 
    frames and converting frames to notes with time information.
    """
    def __init__(self, trackNum=0, frameRate=FRAMERATE, filePath=os.path.join(BASE_PATH, 'GuitarSet')):
        """
        Parameters:
            trackNum (int): The index of the track to load from the dataset.
            frameRate (float, optional): The frame rate of the tab sequence. Default is FRAMERATE.
            filePath (str, optional): The file path to the GuitarSet dataset. Default is 'BASE_PATH/GuitarSet'.
            """
        super().__init__(frameRate)

        self.filePath = filePath
        self.guitarset = mirdata.initialize(
            'guitarset', data_home=self.filePath)

        self.guitarsetIds = self.guitarset.track_ids  # the list of guitar's track ids
        self.guitarsetData = self.guitarset.load_tracks()  # Load all tracks in the dataset
        # Get the first track
        self.track = self.guitarsetData[self.guitarsetIds[trackNum]]
        # trackNum starts at 0

        # strings and their Midi
        self.Elow = 40
        self.A = 45
        self.D = 50
        self.G = 55
        self.B = 59
        self.EHigh = 64

        self.EStringFrets = Functions.midiToFret(
            self.Elow, self.track.notes["E"].pitches)
        self.ETimeStamp = self.track.notes['E'].intervals
        self.AStringFrets = Functions.midiToFret(
            self.A, self.track.notes["A"].pitches)
        self.ATimeStamp = self.track.notes['A'].intervals
        self.DStringFrets = Functions.midiToFret(
            self.D, self.track.notes["D"].pitches)
        self.DTimeStamp = self.track.notes['D'].intervals
        self.GStringFrets = Functions.midiToFret(
            self.G, self.track.notes["G"].pitches)
        self.GTimeStamp = self.track.notes['G'].intervals
        self.BStringFrets = Functions.midiToFret(
            self.B, self.track.notes["B"].pitches)
        self.BTimeStamp = self.track.notes['B'].intervals
        self.eStringFrets = Functions.midiToFret(
            self.EHigh, self.track.notes["e"].pitches)
        self.eTimeStamp = self.track.notes['e'].intervals

        # guitarSet is automatically in midi notes
        self.EMidiPitches = self.track.notes["E"].pitches
        self.AMidiPitches = self.track.notes["A"].pitches
        self.DMidiPitches = self.track.notes["D"].pitches
        self.GMidiPitches = self.track.notes["G"].pitches
        self.BMidiPitches = self.track.notes["B"].pitches
        self.eMidiPitches = self.track.notes["e"].pitches

        self.maxEndTime = max(self.ETimeStamp[-1][-1], self.ATimeStamp[-1][-1],
                              self.DTimeStamp[-1][-1], self.GTimeStamp[-1][-1], self.BTimeStamp[-1][-1], self.eTimeStamp[-1][-1])

        self.numOfStrings = 6
        self.maxFrames = math.ceil(self.maxEndTime) * self.frameRate

        self.fretFrames = []
        self.midiFrames = []
        self.notesWithTimeFrames = []
        self.frameType = "fret"

    def getNumOfStrings(self):
        """
        Get the number of strings in the tab sequence.

        Returns:
            int: The number of strings.
        """
        return self.numOfStrings

    def setNumOfStrings(self, strings):
        """
        Set the number of strings in the tab sequence.

        Parameters:
            strings (int): The number of strings to set.
        """
        self.numOfStrings = strings
    
    def getMaxFrames(self):
        """
        Get the maximum number of frames in the tab sequence.

        Returns:
            int: The maximum number of frames.
        """
        return math.ceil(self.getMaxEndTime()) * self.getFrameRate()

    def setMaxFrames(self, maxFrames):
        """
        Set the maximum number of frames in the tab sequence.

        Parameters:
            maxFrames (int): The maximum number of frames to set.
        """
        self.maxFrames = maxFrames
    
    def getMaxEndTime(self):
        """
        Get the maximum end time of the tab sequence.

        Returns:
            float: The maximum end time.
        """
        return self.maxEndTime

    def setMaxEndTime(self, maxEndTime):
        """
        Set the maximum end time of the tab sequence.

        Parameters:
            maxEndTime (float): The maximum end time to set.
        """
        self.maxEndTime = maxEndTime

    def getStringFrets(self):
        """
        Get the fret positions of each string in the tab sequence.

        Returns:
            list: A list of lists representing the fret positions of each string.
        """
        return [self.EStringFrets, self.AStringFrets, self.DStringFrets, self.GStringFrets, self.BStringFrets, self.eStringFrets]

    def getStringMidi(self):
        """
        Get the MIDI pitches of each string in the tab sequence.

        Returns:
            list: A list of MIDI pitches of each string.
        """
        return [self.Elow, self.A, self.D, self.G, self.B, self.EHigh]
    
    def getMidiPitches(self):
        """
        Get the MIDI pitches of all strings in the tab sequence.

        Returns:
            list: A list of lists representing the MIDI pitches of all strings.
        """
        arr = [self.EMidiPitches, self.AMidiPitches, self.DMidiPitches,
               self.GMidiPitches, self.BMidiPitches, self.eMidiPitches]
        roundedArray = [[round(num) for num in innerArr] for innerArr in arr]

        return roundedArray

    def makingFrames(self):
        """
        Create the frames for the tab sequence based on the string frets or MIDI pitches.

        This method populates the `fretFrames` attribute if the frame type is "fret",
        otherwise it populates the `midiFrames` attribute.

        Note: The method assumes that the frets or MIDI pitches have already been initialized.

        """
        frames = [['x'] * self.numOfStrings for _ in range(self.getMaxFrames())]
        pitchesArr = self.getStringFrets() if self.frameType == "fret" else self.getMidiPitches()

        for i, (pitch, time) in enumerate(zip(pitchesArr,
                                              [self.ETimeStamp, self.ATimeStamp, self.DTimeStamp,
                                               self.GTimeStamp, self.BTimeStamp, self.eTimeStamp])):
            for j in range(len(pitch)):
                startFrame = round(time[j][0] * self.frameRate)
                endFrame = round(time[j][1] * self.frameRate)
                for frame in range(startFrame, endFrame):
                    frames[frame][i] = pitch[j]

        if self.frameType == "fret":
            frames = [",".join([str(num) for num in frame]) for frame in frames]
            self.setFretFrames(frames)
        
        else:
            self.setMidiFrames(frames)

    def framesToNotesWithTime(self):
        """
        Convert the frames to video notes with time information.

        This method populates the `notesWithTimeFrames` attribute based on the `midiFrames` attribute.
        The video notes represent the notes being played along with their start and end times.
        """
        pitchesPlaying = {}
        output = []

        for i, frame in enumerate(self.midiFrames):

            for j, pitch in enumerate(frame):
                if pitch != 'x':
                    # Note being played
                    if (j, pitch) in pitchesPlaying:
                        # Note is already being played
                        # not should be string index and the note pitch
                        pitchesPlaying[(j, pitch)].setEndTime(pitchesPlaying[(j, pitch)].getEndTime() + self.getFramePeriod())
                    else:
                        # Note is starting to be played
                        pitchesPlaying[(j, pitch)] = VideoNote(pitch,i / self.frameRate,i / self.frameRate)

            # Loop through notes currently being played and check if it's in current frame, if not end time
            for (j, pitch) in list(pitchesPlaying):
                if (i == len(self.midiFrames)-1) or (pitch != self.midiFrames[i+1][j]):
                    output.append(pitchesPlaying[(j, pitch)])
                    del pitchesPlaying[(j, pitch)]

        self.setNotesWithTimeFrames(output)

    def addTab(self, seconds, tab):
        """
        Add a tab to the tab sequence.

        Parameters:
            seconds (float): The duration of the tab in seconds.
            tab (str or list): The tab to add. If it's a string, it represents a single frame.
                               If it's a list, it represents multiple frames.
        """
        frames = seconds * self.frameRate
        for i in range(1, frames+1):
            self.addFretFrame(tab)

    def getFretFrames(self):
        """
        Get the fret frames of the tab sequence.

        Returns:
            list: A list of strings representing the fret frames.
        """
        return self.fretFrames

    def getFramesAsString(self):
        """
        Get the fret frames of the tab sequence as strings.

        Returns:
            list: A list of strings representing the fret frames.
        """
        stringFrames = [",".join([str(num) for num in sublist]) for sublist in self.fretFrames]
        return stringFrames

    def addFretFrame(self, frame):
        """
        Add a fret frame to the tab sequence.

        Parameters:
            frame (str or list): The fret frame to add. If it's a string, it represents a single frame.
                                 If it's a list, it represents multiple frames.
        """
        self.fretFrames.append(frame)

    def setFretFrames(self, frames):
        """
        Set the fret frames of the tab sequence.

        Parameters:
            frames (list): A list of strings representing the fret frames.
        """
        self.fretFrames = frames

    def getFretFrames(self):
        """
        Get the fret frames of the tab sequence.

        Returns:
            list: A list of strings representing the fret frames.
        """
        return self.fretFrames

    def addMidiFrame(self, frame):
        """
        Add a MIDI frame to the tab sequence.

        Parameters:
            frame (str or list): The MIDI frame to add. If it's a string, it represents a single frame.
                                 If it's a list, it represents multiple frames.
        """
        self.midiFrames.append(frame)

    def setMidiFrames(self, frames):
        """
        Set the MIDI frames of the tab sequence.

        Parameters:
            frames (list): A list of strings representing the MIDI frames.
        """
        self.midiFrames = frames

    def getMidiFrames(self):
        """
        Get the MIDI frames of the tab sequence.

        Returns:
            list: A list of strings representing the MIDI frames.
        """
        return self.midiFrames

    def getNotesWithTimeFrames(self):
        """
        Get the video notes with time information of the tab sequence.

        Returns:
            list: A list of VideoNote objects representing the notes being played along with their start and end times.
        """
        return self.notesWithTimeFrames

    def setNotesWithTimeFrames(self, notes):
        """
        Set the video notes with time information of the tab sequence.

        Parameters:
            notes (list): A list of VideoNote objects representing the notes being played along with their start and end times.
        """
        self.notesWithTimeFrames = notes

    def getFrameType(self):
        """
        Get the frame type of the tab sequence.

        Returns:
            str: The frame type, either "fret" or "midi".
        """
        return self.frameType.lower()

    def setFrameType(self, frameType):
        """
        Set the frame type of the tab sequence.

        Parameters:
            frameType (str): The frame type to set, either "fret" or "midi".
        """
        self.frameType = frameType



from bokeh.plotting import figure, show
from bokeh.models import Line, Circle, Label, Button, CustomJS, ColumnDataSource, PanTool, BoxSelectTool, Range1d
from bokeh.models.widgets import TextInput
from bokeh.layouts import layout
from bokeh.events import ButtonClick
from bokeh.io import  curdoc
from bokeh.layouts import row
from bokeh.resources import CDN
from bokeh.embed import file_html, components, server_document
from bokeh.application import Application
from bokeh.application.handlers import FunctionHandler

from seeFretboard.Designs.CirlceNote import CircleNote
from seeFretboard.Utilities import Functions
import seeFretboard.Utilities.Constants as Constants
from seeFretboard.PitchCollection import PitchCollection
from seeFretboard.Designs.FretboardStyle import *
from seeFretboard.Designs.FretboardFigure import FretboardFigure
from seeFretboard.NotePosition import NotePositionsOnCurrentFretboard
from seeFretboard.Utilities.PathInfo import EmbedPathInfo
from seeFretboard.Utilities import Constants

from tqdm import tqdm

from music21 import scale, interval, harmony, key
from music21 import pitch as m21Pitch
#manager/master kinda
class SeeFretboard():
    
    # default values
    #fret 0 include open strings
    def __init__(self, orientation = "h",fretFrom=1, fretTo=12, string=6, showTuning=True, theme="", **kwargs):
        # styleing/theme
        self.fretboardTheme = FretboardTheme(theme=theme, orientation=orientation, 
                                    fretFrom =fretFrom, fretTo=fretTo, string=string,
                                    showTuning=showTuning, **kwargs)

        # notes circle
        self.currentNotesPositionOnFretboard = [] #NotePosition
        self.notes = [] #glyph of the current notes more for removeable purposes
        self.labels = []
        
        # figure attribute
        self.fretboardFig = FretboardFigure( self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue(), self.fretboardTheme, orientation)
        
        self.stringSource = ""
        self.fretSource = ""

        # buttons
        self.tuningLabelButton = Button(
            label="Toggle Tuning", button_type="success")
        self.fretLabelButton = Button(
            label="Toggle Fretboard Number", button_type="success")
        self.fretBoardDirectionButton = Button(
            label="Toggle Fretboard Direction", button_type="success")
        self.toggleButtons = row(
            self.fretBoardDirectionButton, self.tuningLabelButton, self.fretLabelButton)
        self.inputChordInput = TextInput(
            value="x,0,2,2,0,0", title="Enter Notes Fret:")
        self.inputChordButton = Button(label="ENTER ", button_type="success")
        self.clearFretboardButton = Button(
            label="Clear Fretboard ", button_type="success")
        self.notesOptions = row(
            self.inputChordInput, self.inputChordButton, self.clearFretboardButton)

        self.inputChordButton.on_click(self.inputChordButtonClicked)
        self.clearFretboardButton.on_click(self.clearFretboard)
        self.fretBoardDirectionButton.on_click(self.toggleFretboardDirection)

        #notes/pitches for scales n arpeggios
        self.pitchCollection = PitchCollection()
        
        self.scaleCustom = False

        self.pathInfo = EmbedPathInfo()

        self.layout = layout([self.fretboardFig.fig,
                         self.toggleButtons, self.notesOptions])

    def getFretboardFig(self):
        return self.fretboardFig
    
    def setFretboardFig(self, fig):
        self.fretboardFig = fig

    def inputChordButtonClicked(self):
        #setting pitch collection let me think think
        self.updateFretboard(self.inputChordInput.value)

    # fretboard relate
    def drawTuningLabel(self, distanceStrings, i):
        if (self.fretboardTheme.orientation.orientation == "h"):
            stringLabel = Label(x=-1, y=distanceStrings-self.fretboardTheme.fretboardDesign.distanceBetweenStrings,
                                 text=self.fretboardTheme.tuning.letterTuning[i], text_align='center', text_font_size=self.fretboardTheme.fretboardDesign.tuningLabelFontSize, text_font_style='bold', text_color=self.fretboardTheme.fretboardDesign.tuningLabelColor)
        else:
            stringLabel = Label(x=distanceStrings, y=self.fretboardTheme.fretboardDesign.distanceBetweenFrets*(
                                self.fretboardTheme.fretboardRange.numOfFrets+1), text=self.fretboardTheme.tuning.letterTuning[i+1], text_align='center', text_font_size=self.fretboardTheme.fretboardDesign.tuningLabelFontSize, text_color=self.fretboardTheme.fretboardDesign.tuningLabelColor, text_font_style='bold')

        stringLabel.visible = self.fretboardTheme.fretboardDesign.showTuning
        
        self.fretboardFig.stringLabel = stringLabel
        self.fretboardFig.addStringLabelLayout()

        self.tuningLabelButton.js_on_event(ButtonClick, CustomJS(args=dict(
            stringLabel=stringLabel), code="""stringLabel.visible = !stringLabel.visible"""))

    def drawFretLabel(self, distanceBetweenFrets, j):
        if (self.fretboardTheme.orientation.orientation == "h"):
            fretLabel = Label(x=distanceBetweenFrets+self.fretboardTheme.fretboardDesign.distanceBetweenFrets-self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                               y=-self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius*2.5, text=str(j+1), text_align='center', text_font_size=self.fretboardTheme.fretboardDesign.tuningLabelFontSize, text_color=self.fretboardTheme.fretboardDesign.tuningLabelColor,text_font_style='bold')
        else:
            fretLabel = Label(x=-self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius*2.5, y=distanceBetweenFrets+self.fretboardTheme.fretboardDesign.distanceBetweenFrets -
                               self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2, text=str(j), text_align='center', text_font_size=self.fretboardTheme.fretboardDesign.tuningLabelFontSize, text_color=self.fretboardTheme.fretboardDesign.tuningLabelColor,text_font_style='bold')

        fretLabel.visible = self.fretboardTheme.fretboardDesign.showFretboardNumber
        
        self.fretboardFig.fretLabel = fretLabel
        self.fretboardFig.addFretLabelLayout()

        self.fretLabelButton.js_on_event(ButtonClick, CustomJS(args=dict(
            fretLabel=fretLabel), code="""fretLabel.visible = !fretLabel.visible"""))

    def toggleFretboardDirection(self):
        notesPosOnFretboard = self.getCurrentNotesOnFretboard()
        self.removeFigure()

        if(self.fretboardTheme.orientation.orientation in Constants.HORIZONTAL):
            self.setFretboardFig(FretboardFigure(self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue(), self.getTheme(), "v"))
            self.getTheme().orientation.orientation = "v"
            self.drawVerticalFretboard()

        else:
            self.setFretboardFig(FretboardFigure(self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue(), self.getTheme(), "h"))
            self.getTheme().orientation.orientation = "h"
            self.drawHorizontalFretboard()

        self.layout.children.insert(0,self.fretboardFig.fig)

        self.notes = []
        self.labels = []

        # Add the progress bar
        with tqdm(total=len(notesPosOnFretboard)) as pbar:
            for i, notePos in enumerate(notesPosOnFretboard):
                self.pitchCollection.setPitchesIndex(i)
                self.addNote(self.pitchCollection.getStringsAt(i),self.pitchCollection.getFretsAt(i))
                pbar.update(1)  # Update the progress bar
        print("Toggle Fretboard Success")
        
    def drawFretboard(self, orientation):
        if(orientation.lower() in Constants.HORIZONTAL):
            self.fretboardTheme.orientation.orientation = "h"
            self.fretboardFig.switchOrientation("h")
            self.drawHorizontalFretboard()
        elif(orientation.lower() in Constants.VERTICAL):
            self.fretboardTheme.orientation.orientation = "v"
            self.fretboardFig.switchOrientation("v")
            self.drawVerticalFretboard()

    # preview
    def drawHorizontalFretboard(self):
        distanceStrings = 0        
        strings = {
            "x": [],
            "y": []
        }
        frets = {
            "x": [],
            "y": []
        }
        # draw strings (horizontal line)
        for i in range(0, self.fretboardTheme.tuning.numOfStrings):
            strings["x"].append([0, self.fretboardTheme.fretboardDesign.distanceBetweenFrets * (self.fretboardTheme.fretboardRange.fretTo - self.fretboardTheme.fretboardRange.fretFrom + 1)])
            strings["y"].append([distanceStrings, distanceStrings])

            self.drawTuningLabel(distanceStrings + self.fretboardTheme.fretboardDesign.distanceBetweenStrings, i)

            distanceStrings += self.fretboardTheme.fretboardDesign.distanceBetweenStrings
        self.stringSource = ColumnDataSource(data=strings)
        self.fretboardFig.fig.multi_line(xs="x", ys="y", line_color=self.fretboardTheme.fretboardDesign.stringsColor,
                                   line_alpha=self.fretboardTheme.fretboardDesign.stringsOpacity, line_width = self.fretboardTheme.fretboardDesign.stringThinkness, source=self.stringSource)

        distanceBetweenFrets = 0
    
        # draw frets (vertical line)
        for j in range(self.fretboardTheme.fretboardRange.fretFrom - 1, self.fretboardTheme.fretboardRange.fretTo + 1):
            frets["x"].append([distanceBetweenFrets, distanceBetweenFrets])
            frets["y"].append([0, self.fretboardTheme.fretboardDesign.distanceBetweenStrings * (self.fretboardTheme.tuning.numOfStrings - 1)])

            if j != self.fretboardTheme.fretboardRange.fretTo:
                self.drawFretLabel(distanceBetweenFrets, j)

            distanceBetweenFrets += self.fretboardTheme.fretboardDesign.distanceBetweenFrets

        self.fretSource = ColumnDataSource(data=frets)
        self.fretboardFig.fig.multi_line(xs="x", ys="y", line_color=self.fretboardTheme.fretboardDesign.fretColor,
                                        line_alpha=self.fretboardTheme.fretboardDesign.fretOpacity, line_width = self.fretboardTheme.fretboardDesign.fretThinkness, source=self.fretSource)
        self.drawInlay()

        self.fretboardFig.fig.y_range=Range1d(0-self.fretboardTheme.fretboardDesign.distanceBetweenStrings, self.fretboardTheme.fretboardDesign.distanceBetweenStrings*self.fretboardTheme.fretboardRange.numOfStrings,bounds='auto')
        self.fretboardFig.fig.background_fill_color = self.fretboardTheme.fretboardDesign.backgroundColor

    def drawVerticalFretboard(self):
        x = [0, 0]
        y = [0, self.fretboardTheme.fretboardDesign.distanceBetweenFrets*(self.fretboardTheme.fretboardRange.numOfFrets+1)]

        self.drawTuningLabel(0, -1)

        self.fretboardFig.fig.line(x=x, y=y, line_color=self.fretboardTheme.fretboardDesign.stringsColor,
                      line_alpha=self.fretboardTheme.fretboardDesign.stringsOpacity)

        distanceStrings = self.fretboardTheme.fretboardDesign.distanceBetweenStrings
        # draw strings (vertical line)
        for i in range(0, self.fretboardTheme.tuning.numOfStrings-1):
            x = [distanceStrings, distanceStrings]
            y = [0, self.fretboardTheme.fretboardDesign.distanceBetweenFrets*(self.fretboardTheme.fretboardRange.numOfFrets+1)]

            self.drawTuningLabel(distanceStrings, i)

            distanceStrings += self.fretboardTheme.fretboardDesign.distanceBetweenStrings
            self.fretboardFig.fig.line(x=x, y=y, line_color=self.fretboardTheme.fretboardDesign.stringsColor,
                          line_alpha=self.fretboardTheme.fretboardDesign.stringsOpacity, line_width = self.fretboardTheme.fretboardDesign.fretThinkness)

        distanceBetweenFrets = (self.fretboardTheme.fretboardRange.numOfFrets+1) * \
            self.fretboardTheme.fretboardDesign.distanceBetweenFrets

        fx = [0, self.fretboardTheme.fretboardDesign.distanceBetweenStrings*(self.fretboardTheme.tuning.numOfStrings-1)]
        fy = [0, 0]
        self.fretboardFig.fig.line(x=fx, y=fy, line_color=self.fretboardTheme.fretboardDesign.fretColor,
                      line_alpha=self.fretboardTheme.fretboardDesign.fretOpacity , line_width = self.fretboardTheme.fretboardDesign.stringThinkness)

        # draw frets (horizontal line)
        fretlength = self.fretboardTheme.fretboardRange.fretFrom-1

        for j in range(self.fretboardTheme.fretboardRange.fretFrom, self.fretboardTheme.fretboardRange.fretTo+1):
            fx = [0, self.fretboardTheme.fretboardDesign.distanceBetweenStrings*(self.fretboardTheme.tuning.numOfStrings-1)]
            fy = [distanceBetweenFrets, distanceBetweenFrets]

            if (j != self.fretboardTheme.fretboardRange.fretFrom):
                self.drawFretLabel(distanceBetweenFrets, fretlength)

            fretlength += 1
            distanceBetweenFrets -= self.fretboardTheme.fretboardDesign.distanceBetweenFrets
            self.fretboardFig.fig.line(x=fx, y=fy, line_color=self.fretboardTheme.fretboardDesign.fretColor,
                          line_alpha=self.fretboardTheme.fretboardDesign.fretOpacity, line_width = self.fretboardTheme.fretboardDesign.fretThinkness,)

        self.drawFretLabel(distanceBetweenFrets, fretlength)

        self.drawInlay()                

        self.fretboardFig.fig.x_range=Range1d(0-self.fretboardTheme.fretboardDesign.distanceBetweenStrings, self.fretboardTheme.fretboardDesign.distanceBetweenStrings*self.fretboardTheme.fretboardRange.numOfStrings,bounds='auto')
        self.fretboardFig.fig.toolbar_location =  self.fretboardTheme.fretboardDesign.toolBar
        
        self.fretboardFig.fig.background_fill_color = self.fretboardTheme.fretboardDesign.backgroundColor

    def drawInlay(self):
        if (self.fretboardTheme.orientation.orientation == "h"):
            # draw 3,5,7,9 marker
            if (self.fretboardTheme.fretboardRange.fretFrom <= 3 <= self.fretboardTheme.fretboardRange.fretTo):
                markerFret3 = self.fretboardFig.fig.circle(x=(3-self.fretboardTheme.fretboardRange.fretFrom+1)*self.fretboardTheme.fretboardDesign.distanceBetweenFrets-self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                                              y=(self.fretboardTheme.tuning.numOfStrings-1) *
                                              self.fretboardTheme.fretboardDesign.distanceBetweenStrings/2,
                                              radius=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius,
                                              fill_color=self.fretboardTheme.fretboardDesign.fretboardMarkerColor,
                                              line_width=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeWidth,
                                              fill_alpha=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteFill,
                                              line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeColor)
            if (self.fretboardTheme.fretboardRange.fretFrom <= 5 <= self.fretboardTheme.fretboardRange.fretTo):
                markerFret5 = self.fretboardFig.fig.circle(x=(5-self.fretboardTheme.fretboardRange.fretFrom+1)*self.fretboardTheme.fretboardDesign.distanceBetweenFrets-self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                                              y=(self.fretboardTheme.tuning.numOfStrings-1) *
                                              self.fretboardTheme.fretboardDesign.distanceBetweenStrings/2,
                                              radius=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius,
                                              fill_color=self.fretboardTheme.fretboardDesign.fretboardMarkerColor,
                                              line_width=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeWidth,
                                              fill_alpha=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteFill,
                                              line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeColor)
            if (self.fretboardTheme.fretboardRange.fretFrom <= 7 <= self.fretboardTheme.fretboardRange.fretTo):
                markerFret7 = self.fretboardFig.fig.circle(x=(7-self.fretboardTheme.fretboardRange.fretFrom+1)*self.fretboardTheme.fretboardDesign.distanceBetweenFrets-self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                                              y=(self.fretboardTheme.tuning.numOfStrings-1) *
                                              self.fretboardTheme.fretboardDesign.distanceBetweenStrings/2,
                                              radius=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius,
                                              fill_color=self.fretboardTheme.fretboardDesign.fretboardMarkerColor,
                                              line_width=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeWidth,
                                              fill_alpha=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteFill,
                                              line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeColor)
            if (self.fretboardTheme.fretboardRange.fretFrom <= 9 <= self.fretboardTheme.fretboardRange.fretTo):
                markerFret9 = self.fretboardFig.fig.circle(x=(9-self.fretboardTheme.fretboardRange.fretFrom+1)*self.fretboardTheme.fretboardDesign.distanceBetweenFrets-self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                                              y=(self.fretboardTheme.tuning.numOfStrings-1) *
                                              self.fretboardTheme.fretboardDesign.distanceBetweenStrings/2,
                                              radius=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius,
                                              fill_color=self.fretboardTheme.fretboardDesign.fretboardMarkerColor,
                                              line_width=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeWidth,
                                              fill_alpha=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteFill,
                                              line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeColor)
            if (self.fretboardTheme.fretboardRange.fretFrom <= 12 <= self.fretboardTheme.fretboardRange.fretTo):
                markerFret12_1 = self.fretboardFig.fig.circle(x=(12-self.fretboardTheme.fretboardRange.fretFrom+1)*self.fretboardTheme.fretboardDesign.distanceBetweenFrets-self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                                                 y=(self.fretboardTheme.tuning.numOfStrings) *
                                                 self.fretboardTheme.fretboardDesign.distanceBetweenStrings/4,
                                                 radius=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius,
                                                 fill_color=self.fretboardTheme.fretboardDesign.fretboardMarkerColor,
                                                 line_width=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeWidth,
                                                 fill_alpha=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteFill,
                                                 line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeColor)
                markerFret12_2 = self.fretboardFig.fig.circle(x=(12-self.fretboardTheme.fretboardRange.fretFrom+1)*self.fretboardTheme.fretboardDesign.distanceBetweenFrets-self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                                                 y=(self.fretboardTheme.tuning.numOfStrings) *
                                                 self.fretboardTheme.fretboardDesign.distanceBetweenStrings/1.75,
                                                 radius=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius,
                                                 fill_color=self.fretboardTheme.fretboardDesign.fretboardMarkerColor,
                                                 line_width=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeWidth,
                                                 fill_alpha=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteFill,
                                                 line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeColor)

        else:
            if (self.fretboardTheme.fretboardRange.fretFrom <= 3 <= self.fretboardTheme.fretboardRange.fretTo):
                markerFret3 = self.fretboardFig.fig.circle(x=(self.fretboardTheme.tuning.numOfStrings-1)*self.fretboardTheme.fretboardDesign.distanceBetweenStrings/2,
                                              y=self.fretboardTheme.fretboardDesign.distanceBetweenFrets*(self.fretboardTheme.fretboardRange.numOfFrets) - self.fretboardTheme.fretboardDesign.distanceBetweenFrets*(
                                                  3-self.fretboardTheme.fretboardRange.fretFrom-1) - self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                                              radius=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius/2,
                                              fill_color=self.fretboardTheme.fretboardDesign.fretboardMarkerColor,
                                              line_width=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeWidth,
                                              fill_alpha=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteFill,
                                              line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeColor)
            if (self.fretboardTheme.fretboardRange.fretFrom <= 5 <= self.fretboardTheme.fretboardRange.fretTo):
                markerFret5 = self.fretboardFig.fig.circle(x=(self.fretboardTheme.tuning.numOfStrings-1)*self.fretboardTheme.fretboardDesign.distanceBetweenStrings/2,
                                              y=self.fretboardTheme.fretboardDesign.distanceBetweenFrets*(self.fretboardTheme.fretboardRange.numOfFrets) - self.fretboardTheme.fretboardDesign.distanceBetweenFrets*(
                                                  5-self.fretboardTheme.fretboardRange.fretFrom-1) - self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                                              radius=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius/2,
                                              fill_color=self.fretboardTheme.fretboardDesign.fretboardMarkerColor,
                                              line_width=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeWidth,
                                              fill_alpha=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteFill,
                                              line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeColor)
            if (self.fretboardTheme.fretboardRange.fretFrom <= 7 <= self.fretboardTheme.fretboardRange.fretTo):
                markerFret7 = self.fretboardFig.fig.circle(x=(self.fretboardTheme.tuning.numOfStrings-1)*self.fretboardTheme.fretboardDesign.distanceBetweenStrings/2,
                                              y=self.fretboardTheme.fretboardDesign.distanceBetweenFrets*(self.fretboardTheme.fretboardRange.numOfFrets) - self.fretboardTheme.fretboardDesign.distanceBetweenFrets*(
                                                  7-self.fretboardTheme.fretboardRange.fretFrom-1) - self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                                              radius=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius/2,
                                              fill_color=self.fretboardTheme.fretboardDesign.fretboardMarkerColor,
                                              line_width=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeWidth,
                                              fill_alpha=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteFill,
                                              line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeColor)
            if (self.fretboardTheme.fretboardRange.fretFrom <= 9 <= self.fretboardTheme.fretboardRange.fretTo):
                markerFret9 = self.fretboardFig.fig.circle(x=(self.fretboardTheme.tuning.numOfStrings-1)*self.fretboardTheme.fretboardDesign.distanceBetweenStrings/2,
                                              y=self.fretboardTheme.fretboardDesign.distanceBetweenFrets*(self.fretboardTheme.fretboardRange.numOfFrets) - self.fretboardTheme.fretboardDesign.distanceBetweenFrets*(
                                                  9-self.fretboardTheme.fretboardRange.fretFrom-1) - self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                                              radius=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius/2,
                                              fill_color=self.fretboardTheme.fretboardDesign.fretboardMarkerColor,
                                              line_width=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeWidth,
                                              fill_alpha=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteFill,
                                              line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeColor)
            if (self.fretboardTheme.fretboardRange.fretFrom <= 12 <= self.fretboardTheme.fretboardRange.fretTo):
                markerFret12_1 = self.fretboardFig.fig.circle(x=(self.fretboardTheme.tuning.numOfStrings)*self.fretboardTheme.fretboardDesign.distanceBetweenStrings*2/3-self.fretboardTheme.fretboardDesign.distanceBetweenStrings/2,
                                                 y=self.fretboardTheme.fretboardDesign.distanceBetweenFrets*(self.fretboardTheme.fretboardRange.numOfFrets) - self.fretboardTheme.fretboardDesign.distanceBetweenFrets*(
                                                     12-self.fretboardTheme.fretboardRange.fretFrom-1) - self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                                                 radius=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius/2,
                                                 fill_color=self.fretboardTheme.fretboardDesign.fretboardMarkerColor,
                                                 line_width=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeWidth,
                                                 fill_alpha=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteFill,
                                                 line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeColor)
                markerFret12_2 = self.fretboardFig.fig.circle(x=(self.fretboardTheme.tuning.numOfStrings)*self.fretboardTheme.fretboardDesign.distanceBetweenStrings/3-self.fretboardTheme.fretboardDesign.distanceBetweenStrings/2,
                                                 y=self.fretboardTheme.fretboardDesign.distanceBetweenFrets*(self.fretboardTheme.fretboardRange.numOfFrets) - self.fretboardTheme.fretboardDesign.distanceBetweenFrets*(
                                                     12-self.fretboardTheme.fretboardRange.fretFrom-1) - self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                                                 radius=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius/2,
                                                 fill_color=self.fretboardTheme.fretboardDesign.fretboardMarkerColor,
                                                 line_width=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeWidth,
                                                 fill_alpha=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteFill,
                                                 line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeColor)
        # self.notes.append(self.fig.add_glyph(circleNote))

    def showFretboard(self):
        
        curdoc().add_root(self.layout)


    def clearFretboard(self):
        notesCopy = list(self.notes)

        for note in notesCopy:
            self.fretboardFig.fig.renderers.remove(note)
            self.notes.remove(note)
        
        for label in self.labels:
            label.text = ""
        
        self.labels = []
        self.currentNotesPositionOnFretboard = []

    def removeFigure(self):
        self.currentNotesPositionOnFretboard = []
        self.layout.children.remove(self.fretboardFig.fig)
        
    def updateFretboard(self, notes):
        self.clearFretboard()
        self.addNotesAllString(notes)

    # user input = string like "1,0,1,1,0,0" which correspond to standard tuning "E,A,D,G,B,E"
    def addNotesAllString(self, notes):
        notes = [(x.strip()) for x in notes.split(',')]
        midiNotes = []
        strings = []
        frets = []
        len(notes)
        self.pitchCollection.setAllEmpty()
        for i in range(len(notes)):
            self.pitchCollection.appendPitchesEmpty("")
            strings.append(i)
            frets.append(notes[i])
            midiNotes.append(str(Functions.fretToMidi(self.fretboardTheme.tuning.midiTuning[i],notes[i])))
        self.pitchCollection.setStrings(strings)
        self.pitchCollection.setFrets(frets)
        notesWithOctaveName = Functions.midisToNoteNameWithOctaves(midiNotes)
        print(notesWithOctaveName)
        self.pitchCollection.setPitchWithOctave(notesWithOctaveName)
        
        
        if (len(notes) == self.fretboardTheme.tuning.numOfStrings):
            for i in range(0, self.fretboardTheme.tuning.numOfStrings):
                self.pitchCollection.setPitchesIndex(i)
                self.addNote(i, notes[i])
        else:
            print("ERROR, WRONG FORMAT.")

    # -1 = x
    def addNote(self, string, fret):
        note = ""
        
        textValue = str(self.pitchCollection.getArrayTypeNowAt(self.pitchCollection.getCurrentPitchesIndex()))
        textValue = textValue.replace("-","b")

        notePos = NotePositionsOnCurrentFretboard(string, fret)
        self.appendCurrentNotesOnFretboard(notePos)

        if(fret=="" or fret==''):
            return None

        elif isinstance(fret, str) and fret.lower() == 'x':
            print("")

        elif(int(fret) > self.fretboardTheme.fretboardRange.fretTo or int(fret) < self.fretboardTheme.fretboardRange.fretFrom and (int(fret) != 0)):
            print("fret",fret,"must be within the fretboard range")
            return None

        elif (fret != "0"):
            fret = int(fret)-self.fretboardTheme.fretboardRange.fretFrom+1

        if (self.fretboardTheme.orientation.orientation == "h"):  # edit later
            if (fret == "0"):
                fret = int(fret)
                note = Circle(x=(fret)*self.fretboardTheme.fretboardDesign.distanceBetweenFrets-self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                              y=(string)*self.fretboardTheme.fretboardDesign.distanceBetweenStrings,
                              radius=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius*2.2,
                              fill_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteFaceColor,
                              line_width=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeWidth,
                              line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeColor,
                              fill_alpha = self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteOpactiy,
                              name="circleNote"
                              )
                
                label = Label(x=(fret)*self.fretboardTheme.fretboardDesign.distanceBetweenFrets-self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                              y=(string)*self.fretboardTheme.fretboardDesign.distanceBetweenStrings-self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius/1.5,
                                        text=textValue, text_align='center',text_font_size=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().getNoteTextFont(),text_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().getNoteTextColor())
                self.labels.append(label)
                self.fretboardFig.fig.add_layout(label)
                
            elif (isinstance(fret, str) and fret.lower() == 'x'):
                fret = 0
                xPos = (fret)*self.fretboardTheme.fretboardDesign.distanceBetweenFrets-self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2
                yPos = (string)*self.fretboardTheme.fretboardDesign.distanceBetweenStrings+self.fretboardTheme.fretboardDesign.distanceBetweenStrings/6
                symbolSize = self.fretboardTheme.fretboardDesign.distanceBetweenStrings/8
                
                xCor = [xPos - symbolSize*4, xPos + symbolSize]
                yCor = [yPos - symbolSize, yPos + symbolSize]
                source = ColumnDataSource(data=dict(x=xCor, y=yCor))

                lineOne = Line(x="x",
                               y="y", line_width=3, line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().getNoteFaceColor())
                yCorFlip = yCor[::-1]
                lineTwo = Line(x="x",
                               y="yFlip", line_width=3, line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().getNoteFaceColor())

                source.data["yFlip"] = yCorFlip

                self.notes.append(self.fretboardFig.fig.add_glyph(
                    source, lineOne))
                self.notes.append(self.fretboardFig.fig.add_glyph(
                    source, lineTwo))
                    
                
            else:
                fret = int(fret)
                note = Circle(x=(fret)*self.fretboardTheme.fretboardDesign.distanceBetweenFrets-self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                              y=(string)*self.fretboardTheme.fretboardDesign.distanceBetweenStrings,
                              radius=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius*2.2,
                              fill_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteFaceColor,
                              line_width=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeWidth,
                              line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeColor,
                              fill_alpha = self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteOpactiy,
                              name="circleNote"
                              )
                
                label = Label(x=(fret)*self.fretboardTheme.fretboardDesign.distanceBetweenFrets-self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                              y=(string)*self.fretboardTheme.fretboardDesign.distanceBetweenStrings-self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius/1.5,
                                        text=textValue, text_align='center', text_font_size=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().getNoteTextFont(),text_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().getNoteTextColor())
                self.labels.append(label)
                self.fretboardFig.fig.add_layout(label)
        else:
            if (fret == "0"):
                fret = int(fret)
                note = Circle(x=(string)*self.fretboardTheme.fretboardDesign.distanceBetweenStrings,
                              y=self.fretboardTheme.fretboardDesign.distanceBetweenFrets *
                              (self.fretboardTheme.fretboardRange.numOfFrets+1) +
                              self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().getNoteRadius()*6,
                              radius=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius,
                              fill_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteFaceColor,
                              line_width=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeWidth,
                              line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeColor,
                              fill_alpha = self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteOpactiy,
                              name="circleNote"
                              )
                label = Label(x=(string)*self.fretboardTheme.fretboardDesign.distanceBetweenStrings,
                              y=self.fretboardTheme.fretboardDesign.distanceBetweenFrets *
                              (self.fretboardTheme.fretboardRange.numOfFrets+2) - (fret) *
                              self.fretboardTheme.fretboardDesign.distanceBetweenFrets - self.fretboardTheme.fretboardDesign.distanceBetweenFrets/1.7-self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius/2,
                                        text=textValue, text_align='center', text_font_size=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().getNoteTextFont(),text_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().getNoteTextColor())
                self.labels.append(label)
                self.fretboardFig.fig.add_layout(label)

            elif (isinstance(fret, str) and fret.lower() == 'x'):
                fret = 0
                xPos = (string)*self.fretboardTheme.fretboardDesign.distanceBetweenStrings
                yPos = self.fretboardTheme.fretboardDesign.distanceBetweenFrets * \
                    (self.fretboardTheme.fretboardRange.numOfFrets+1)+self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().getNoteRadius()*6
                symbolSize = self.fretboardTheme.fretboardDesign.distanceBetweenStrings/8

                xCor = [xPos - symbolSize, xPos + symbolSize]
                yCor = [yPos - symbolSize*4, yPos + symbolSize]
                source = ColumnDataSource(data=dict(x=xCor, y=yCor))

                lineOne = Line(x="x",
                               y="y", line_width=3, line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().getNoteFaceColor())
                yCorFlip = yCor[::-1]
                lineTwo = Line(x="x",
                               y="yFlip", line_width=3, line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().getNoteFaceColor())

                source.data["yFlip"] = yCorFlip

                self.notes.append(self.fretboardFig.fig.add_glyph(
                    source, lineOne))
                self.notes.append(self.fretboardFig.fig.add_glyph(
                    source, lineTwo))
                
                label = Label(x=xPos,
                              y=yPos,
                                        text="", text_align='center', text_font_size='10pt')
                self.labels.append(label)

                self.fretboardFig.fig.add_layout(label)

            else:
                fret = int(fret)
                note = Circle(x=(string)*self.fretboardTheme.fretboardDesign.distanceBetweenStrings,
                              y=self.fretboardTheme.fretboardDesign.distanceBetweenFrets *
                              (self.fretboardTheme.fretboardRange.numOfFrets+2) - (fret) *
                              self.fretboardTheme.fretboardDesign.distanceBetweenFrets - self.fretboardTheme.fretboardDesign.distanceBetweenFrets/2,
                              radius=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius,
                              fill_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteFaceColor,
                              line_width=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeWidth,
                              line_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteEdgeColor,
                              fill_alpha = self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteOpactiy,
                              name="circleNote"
                              )
                
                label = Label(x=(string)*self.fretboardTheme.fretboardDesign.distanceBetweenStrings,
                              y=self.fretboardTheme.fretboardDesign.distanceBetweenFrets *
                              (self.fretboardTheme.fretboardRange.numOfFrets+2) - (fret) *
                              self.fretboardTheme.fretboardDesign.distanceBetweenFrets - self.fretboardTheme.fretboardDesign.distanceBetweenFrets/1.7-self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().noteRadius/2,
                                        text=textValue, text_align='center', text_font_size=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().getNoteTextFont(),text_color=self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().getNoteTextColor())
                self.labels.append(label)
                self.fretboardFig.fig.add_layout(label)
                
        if (note != ""):
            self.notes.append(self.fretboardFig.fig.add_glyph(note))
        
    #drawing fretboard getter n setters
    def getTheme(self):
        return self.fretboardTheme
    
    def setTheme(self, theme):
        self.fretboardTheme = theme

    def getCurrentNotesOnFretboard(self):
        return self.currentNotesPositionOnFretboard

    def appendCurrentNotesOnFretboard(self, value):
        self.currentNotesPositionOnFretboard.append(value)

    def setCurrentNotesOnFretboard(self, notes):
        self.currentNotesPositionOnFretboard = notes
    
    #adding scale, arpeggio, interval, chord
    
    #rootNote = "c", type="major"
    def addScale(self, rootNote, type, intervalsDegrees=None):
        
        if type.lower() == 'major' or type.lower() == 'ionian':
            scaleObj = scale.MajorScale(rootNote)
        elif type.lower() == 'minor' or type.lower() == 'aeolian':
            scaleObj = scale.MinorScale(rootNote)
        elif type.lower() == 'harmonic minor':
            scaleObj = scale.HarmonicMinorScale(rootNote)
        elif type.lower() == 'melodic minor':
            scaleObj = scale.MelodicMinorScale(rootNote)
        elif type.lower() == 'dorian':
            scaleObj = scale.DorianScale(rootNote)
        elif type.lower() == 'phrygian':
            scaleObj = scale.PhrygianScale(rootNote)
        elif type.lower() == 'lydian':
            scaleObj = scale.LydianScale(rootNote)
        elif type.lower() == 'mixolydian':
            scaleObj = scale.MixolydianScale(rootNote)
        elif type.lower() == 'locrian':
            scaleObj = scale.LocrianScale(rootNote)
        elif type.lower() == 'blues':
            scaleObj = scale.BluesScale(rootNote)
        elif type.lower() == 'whole tone':
            scaleObj = scale.WholeToneScale(rootNote)
        elif type.lower() == 'chromatic':
            scaleObj = scale.ChromaticScale(rootNote)
        elif type.lower() == 'hypodorian':
            scaleObj = scale.HypodorianScale(rootNote)
        elif type.lower() == 'hypophrygian':
            scaleObj = scale.HypophrygianScale(rootNote)
        elif type.lower() == 'hypolydian':
            scaleObj = scale.HypolydianScale(rootNote)
        elif type.lower() == 'hypomixolydian':
            scaleObj = scale.HypomixolydianScale(rootNote)
        elif type.lower() == 'hypoaeolian':
            scaleObj = scale.HypoaeolianScale(rootNote)
        elif type.lower() == 'octatonic':
            scaleObj = scale.OctatonicScale(rootNote)
        elif type.lower() == 'octaverepeating':
            scaleObj = scale.OctaveRepeatingScale(rootNote)
        elif type.lower() == 'cyclical':
            scaleObj = scale.CyclicalScale(rootNote)
        elif type.lower() == 'ragasawari':
            scaleObj = scale.RagAsawari(rootNote)
        elif type.lower() == 'ragmarwa':
            scaleObj = scale.RagMarwa(rootNote)
        elif type.lower() == 'weightedhexatonicblues':
            scaleObj = scale.WeightedHexatonicBlues(rootNote)
        elif type.lower() == "minorpentatonic":
            intervals = 'P1 m3 P4 P5 m7'.split()
            intervals.append("P1")
            self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().setIntervals(intervals)
            scalePitches = [m21Pitch.Pitch(rootNote).transpose(interval) for interval in intervals]
            self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().setScaleDegrees(Functions.intervalsToScaleDegrees(intervals))
            scaleObj = scale.ConcreteScale(rootNote,scalePitches)
            self.scaleCustom = True

        elif type.lower() == "majorpentatonic":
            intervals = 'P1 M2 M3 P5 M6'.split()
            intervals.append("P1")
            self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().setIntervals(intervals)
            scalePitches = [m21Pitch.Pitch(rootNote).transpose(interval) for interval in intervals]
            self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().setScaleDegrees(Functions.intervalsToScaleDegrees(intervals))
            scaleObj = scale.ConcreteScale(rootNote,scalePitches)
            self.scaleCustom = True
        else:
            # assume user-defined scale
            if intervalsDegrees is None:
                raise ValueError("Intervals must be provided for a user-defined scale.")
            
            intervals = intervalsDegrees.split()
            intervals.append("P1")
            self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().setIntervals(intervals)
            scalePitches = [m21Pitch.Pitch(rootNote).transpose(interval) for interval in intervals]
            self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().setScaleDegrees(Functions.intervalsToScaleDegrees(intervals))
            scaleObj = scale.ConcreteScale(rootNote,scalePitches)
            self.scaleCustom = True
        
        pitches = scaleObj.getPitches()
        #make the arrays or pitches
        self.addPitchesToFretboard(pitches, scaleObj)

    def addPitchesToFretboard(self, pitches, scaleObj):
        self.arraysForPitchCollection(pitches, scaleObj)
        for i in range(len(self.pitchCollection.getArrayTypeNow())):
            self.pitchCollection.setPitchesIndex(i)
            self.addNote(self.pitchCollection.getStringsAt(i),self.pitchCollection.getFretsAt(i))
            

    def arraysForPitchCollection(self, pitches, scaleObj):
        #Add to Arrays for different options
        self.pitchCollection.setPitchesNames([])
        self.pitchCollection.setPitchWithOctave([])
        self.pitchCollection.setPitchesScaleDegrees([])
        self.pitchCollection.setPitchesEmpty([])

        #add to fretboard and octave name string
        for pitchIndex in range(len(pitches)):
            fretNum = []
            stringNum = []

            #get each note on each string n their fret
            fretNum, stringNum= self.convertPitchesToFretsStringsNum(pitches[pitchIndex])    
            
            #depending on how many octave to display 
            quotient, remainder = divmod(self.fretboardTheme.fretboardRange.fretTo+1, 12)
            for octavesFret in range(quotient):
                for index in range(self.fretboardTheme.tuning.numOfStrings):
                    newFret = fretNum[index]+12  
                    fretNum.append(newFret)
                    stringNum.append(index)
            #check to see if between fret range 
            for i in range(len(fretNum)):
                if (self.fretboardTheme.fretboardRange.fretFrom > fretNum[i] or fretNum[i] >= self.fretboardTheme.fretboardRange.fretTo+1) and not((self.fretboardTheme.fretboardRange.fretFrom == 1) and (fretNum[i] == 0)):                    
                    fretNum[i]=""
            # add the notes to the  octave array
            for i in range(len(fretNum)):
                self.pitchCollection.appendFrets(fretNum[i])
                self.pitchCollection.appendStrings(stringNum[i])
                self.pitchCollection.appendPitchesName(pitches[pitchIndex].name)
                self.pitchCollection.appendPitchesEmpty("")


                if(fretNum[i] != ""):
                    midi = Functions.fretToMidi(self.fretboardTheme.tuning.midiTuning[stringNum[i]],fretNum[i])
                    pitchWithOctave = Functions.midiToNoteNameWithOctave(midi)
                else:
                    pitchWithOctave = ""
        
                self.pitchCollection.appendPitchWithOctave(pitchWithOctave)

                #scale degree
                if(self.scaleCustom == True):
                    self.pitchCollection.appendPitchesScaleDegree(self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().getScaleDegrees()[pitchIndex])
        
                else:
                #for nonConcrete scale
                    if(pitches[pitchIndex].alter == -1.0):
                        self.pitchCollection.appendPitchesScaleDegree("b"+str(scaleObj.getScaleDegreeFromPitch(pitches[pitchIndex])))
                    elif(pitches[pitchIndex].alter == 1.0):
                        self.pitchCollection.appendPitchesScaleDegree("#"+str(scaleObj.getScaleDegreeFromPitch(pitches[pitchIndex])))
                    else:
                        self.pitchCollection.appendPitchesScaleDegree(scaleObj.getScaleDegreeFromPitch(pitches[pitchIndex]))

                
            print("NOTE FINSIH")

    #MAYBE PUT somehwere else
    #given pitch, give me all of the places it is located in fret, string
    def convertPitchesToFretsStringsNum(self, pitch):
        #tuning = TabSequence.getStringMidi()#util later
        frets = []
        strings = []
        
        midiPitch=pitch.midi

        for stringIndex, stringPitch in enumerate(self.fretboardTheme.tuning.midiTuning):
            fret = (midiPitch - stringPitch) % 12

            if fret >= (self.fretboardTheme.fretboardRange.fretFrom - 1) and fret <= self.fretboardTheme.fretboardRange.fretTo or fret == 0:
                frets.append(fret)
                strings.append(stringIndex)

        return frets, strings
    
   
    def getArpeggioPitches(self, rootNote, type="", chordPitches="", bass=""):
            if type != "":
                chordObj = harmony.ChordSymbol(root=rootNote, bass=bass, kind=type)
                chordPitches = chordObj.pitches
                intervals = []
                for i in range(len(chordPitches)):
                    intervalObj = interval.Interval(chordPitches[0],chordPitches[i])
                    intervals.append(intervalObj.directedName)
            else:
                intervals = chordPitches.split()
            type = type.lower()    
            intervals.append("P1")
            self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().setIntervals(intervals)
            scalePitches = [m21Pitch.Pitch(rootNote).transpose(interval) for interval in intervals]
            self.fretboardTheme.fretboardDesign.getCurrentNoteTypeValue().setScaleDegrees(Functions.intervalsToScaleDegrees(intervals))
            scaleObj = scale.ConcreteScale(rootNote,scalePitches)
            self.scaleCustom = True
            pitches = scaleObj.getPitches()

            return pitches, scaleObj

    def addArpeggio(self, rootNote, type="", chordPitches="", bass=""):
            pitches, scaleObj = self.getArpeggioPitches(rootNote,type,chordPitches,bass)              

            self.addPitchesToFretboard(pitches, scaleObj)
    
    def getIntervalPitches(self, rootNote, intervalName=""):
        rootNotePitch = m21Pitch.Pitch(rootNote+"2")
        scalePitches = [rootNotePitch.transpose(interval.Interval(intervalName))] 
        scalePitches.insert(0,rootNotePitch)
        scaleObj = scale.ConcreteScale(rootNote,scalePitches)
        pitches = scaleObj.getPitches()
        
        return pitches, scaleObj
    
    def addInterval(self, rootNote, intervalName=""):
            pitches, scaleObj = self.getIntervalPitches(rootNote, intervalName)

            self.addPitchesToFretboard(pitches, scaleObj)


    #there is just so many different ways to think about chord and construct them, a chord works as long as it has the notes of the chord like inversions
    #only work in standard tuning as of now
    #caged dont work well with dim7,aug7 chords, lotta time also just take out 5
    #put in one that is more easier to press
    # Very basic people start with caged
    def addCagedPosChord(self, rootNote, type="", caged="c"):
        chordType = type.lower()
        Functions.checkChordType(chordType)

        Functions.ifInDict(caged, Functions.cagedShapes)

        
        processedShape = Functions.processCAGEDShape(caged.upper(), rootNote, chordType)

        note = processedShape["note"][chordType]
        pos = processedShape["position"][chordType]
        sd = processedShape["scaleDegree"][chordType]

        self.pitchCollection.setFrets(pos)
        self.pitchCollection.setPitchesNames(note)
        self.pitchCollection.setPitchesScaleDegrees(sd)
        self.pitchCollection.setStrings([0,1,2,3,4,5])

        for i in range(len(note)):
            self.pitchCollection.setPitchesIndex(i)
            self.addNote(self.pitchCollection.getStringsAt(i),self.pitchCollection.getFretsAt(i))

    def addDropChord(self, rootNote, type="", drop="Drop2", pos="1", string="6"):
        chordType = type.lower()
        drop = drop.upper()

        Functions.checkChordType(chordType,True)
        Functions.ifInDict(drop, Functions.DROPShapes)

        processedShape = Functions.processDropShape(drop.upper(), rootNote, chordType, string,pos)

        note = processedShape["note"][chordType]
        pos = processedShape["position"][chordType]
        sd = processedShape["scaleDegree"][chordType]

        self.pitchCollection.setFrets(pos)
        self.pitchCollection.setPitchesNames(note)
        self.pitchCollection.setPitchesScaleDegrees(sd)
        self.pitchCollection.setStrings([0,1,2,3,4,5])

        for i in range(len(note)):
            self.pitchCollection.setPitchesIndex(i)
            self.addNote(self.pitchCollection.getStringsAt(i),self.pitchCollection.getFretsAt(i))

    def addCustomShape(self, customShape, name):
        customShape = customShape.getShape(name)
        self.pitchCollection.setFrets(customShape['note'])
        self.pitchCollection.setPitchesNames(customShape['position'])
        self.pitchCollection.setPitchesScaleDegrees(customShape['scaleDegree'])
        self.pitchCollection.setStrings([0,1,2,3,4,5])

        for i in range(len(customShape['note'])):
            self.pitchCollection.setPitchesIndex(i)
            self.addNote(self.pitchCollection.getStringsAt(i),self.pitchCollection.getFretsAt(i))

    def addOctave(self, rootNote):
        self.addArpeggio(rootNote,"","P1")
    
    def getPitchCollection(self):
        return self.pitchCollection
    
    def setPitchCollection(self, pitchCollection):
        self.pitchCollection = pitchCollection

    #Embedding
    def generateStandaloneHtml(self, name="standalone.html"):
        html = file_html(self.getFretboardFig().fig, CDN, "my plot")

        self.pathInfo.name = name
        # Write the HTML to a file
        with open(self.pathInfo.getPathWithName(), "w") as file:
            file.write(html)
    
    def genereateScriptAndDiv(self, scriptName="script.html", divName="div.html"):
        script, div = components(self.fretboardFig.fig)

        self.pathInfo.name = scriptName
        # Write to a file
        with open(self.pathInfo.getPathWithName(), "w") as file:
            file.write(script)
        
        self.pathInfo.name = divName
        with open(self.pathInfo.getPathWithName(), "w") as file:
            file.write(div)

    def generateWithServerHtml(self, name="withServer.html"):
        script = server_document("http://localhost:5006/generalTest")
        
        self.pathInfo.name = name
        with open(self.pathInfo.getPathWithName(), "w") as file:
            file.write(script)




import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from seeFretboard.Videos.TabSequence import TabSequence
from seeFretboard.Videos.Video import Video
from seeFretboard import SeeFretboard

f = SeeFretboard("v", 1, 12)
f.addNotesAllString("'x',0,5,5,0,0")
f.drawVerticalFretboard()

f.video.setAudioName("midiAudio_00_BN1-129-Eb_comp_hex.wav")
tabS = TabSequence(0)
tabS.setFrameRate(60)
tabS.setFrameType("midi")
tabS.makingFrames()
tabS.framesToNotesWithTime()
noteFrames = tabS.getNotesWithTimeFrames()
f.sonifyJams(noteFrames)



import os
from seeFretboard.Utilities.Constants import BASE_PATH

class PathInfo():
    def __init__(self, name, path):
        """
        Initializes a PathInfo object with the specified name and path.

        Args:
            name (str): The name of the path.
            path (str): The path.
        """
        self.path = path
        self.name = name
        self.pathWithName = os.path.join(self.path, self.name)

    @property
    def path(self):
        """
        The path.

        Returns:
            str: The path.
        """
        return self._path

    @path.setter
    def path(self, value):
        """
        Sets the path.

        Args:
            value (str): The path.
        """
        self._path = value

    @property
    def name(self):
        """
        The name of the path.

        Returns:
            str: The name of the path.
        """
        return self._name

    @name.setter
    def name(self, value):
        """
        Sets the name of the path.

        Args:
            value (str): The name of the path.
        """
        self._name = value

    def getPathWithName(self):
        """
        Returns the path concatenated with the name.

        Returns:
            str: The path concatenated with the name.
        """
        return os.path.join(self._path, self._name)

    def setPathWithName(self, value):
        """
        Sets the path and name based on the provided value.

        Args:
            value (str): The value representing the path and name.
        """
        # Split the provided value to separate path and name
        directory, filename = os.path.split(value)
        self._path = directory
        self._name = filename


class EmbedPathInfo(PathInfo):
    def __init__(self, name="default.html"):
        """
        Initializes an EmbedPathInfo object with the specified name and default path.

        Args:
            name (str): The name of the embed path (default is 'default.html').
        """
        self.path = os.path.join(BASE_PATH, 'Outputs', 'Embeds')
        self.name = name
        self.pathWithName = os.path.join(self.path, self.name)



class NotePositionsOnCurrentFretboard():
    """
    The NotePositionsOnCurrentFretboard class represents a position of a note on a guitar fretboard. 
    It has two main attributes: the string number and the fret number. 
    The class provides methods to get and set these attributes.
    """
    def __init__(self, string, fret):
        self.string = string
        self.fret = fret
    
    def getFret(self):
        """
        Returns the fret number of the note position.

        Returns:
            int: The fret number of the note position.
        """
        return self.fret
    
    def setFret(self, fret):
        """
        Sets the fret number of the note position.

        Args:
            fret (int): The fret number of the note position.
        """
        self.fret = fret
    
    def getString(self):
        """
        Returns the string number of the note position.

        Returns:
            int: The string number of the note position.
        """
        return self.string
    
    def setString(self, string):
        """
        Sets the string number of the note position.

        Args:
            string (int): The string number of the note position.
        """
        self.string = string



# Generated by CodiumAI
from seeFretboard.Designs.FretboardStyle import FretboardTheme
from seeFretboard.Designs.FretboardStyle import FretboardOrientation
from seeFretboard.Designs.FretboardStyle import Tuning
from seeFretboard.Designs.FretboardStyle import FretboardRange
from seeFretboard.Designs.FretboardStyle import FretboardDesign
from seeFretboard.Utilities.Constants import STANDARD_TUNING, STANDARD_TUNING_MIDI

import pytest

"""
Code Analysis

Main functionalities:
The FretboardStyle class is responsible for creating a customizable fretboard design. It combines the properties of the FretboardOrientation, Tuning, FretboardRange, and FretboardDesign classes to create a complete fretboard style.

Methods:
The FretboardStyle class does not have any methods of its own. It relies on the methods of the classes it combines to create the fretboard style.

Fields:
- orientation: an instance of the FretboardOrientation class that determines whether the fretboard is displayed horizontally or vertically
- tuning: an instance of the Tuning class that determines the tuning of the strings on the fretboard
- fretboardRange: an instance of the FretboardRange class that determines the range of frets displayed on the fretboard
- fretboardDesign: an instance of the FretboardDesign class that determines the visual design of the fretboard, including the distance between frets and strings, the color of the frets and strings, and whether to display the tuning and fretboard number.
"""

class TestFretboardStyle:
    # Tests that a FretboardStyle object can be created with all valid inputs. 
    def test_create_fretboard_style_with_all_valid_inputs(self):
        style = FretboardTheme(orientation=FretboardOrientation("horizontal"), tuning=Tuning(letterTuning=STANDARD_TUNING), fretboardRange=FretboardRange(1, 12), fretboardDesign=FretboardDesign())
        assert isinstance(style.orientation, FretboardOrientation)
        assert isinstance(style.tuning, Tuning)
        assert isinstance(style.fretboardRange, FretboardRange)
        assert isinstance(style.fretboardDesign, FretboardDesign)

    # Tests that a FretboardStyle object can be created with only some optional inputs. 
    def test_create_fretboard_style_with_some_optional_inputs(self):
        style = FretboardTheme(tuning=Tuning(letterTuning=STANDARD_TUNING), fretboardRange=FretboardRange(1, 12))
        assert isinstance(style.orientation, FretboardOrientation)
        assert isinstance(style.tuning, Tuning)
        assert isinstance(style.fretboardRange, FretboardRange)
        assert isinstance(style.fretboardDesign, FretboardDesign)

    # Tests that a FretboardStyle object cannot be created with an invalid orientation input. 
    def test_create_fretboard_style_with_invalid_orientation_input(self):
        with pytest.raises(ValueError):
            style = FretboardTheme(orientation="invalid")

    # Tests that a FretboardStyle object cannot be created with an invalid tuning input. 
    def test_create_fretboard_style_with_invalid_tuning_input(self):
        with pytest.raises(TypeError):
            style = FretboardTheme(tuning="invalid")

    # Tests that a FretboardStyle object cannot be created with an invalid fretboard range input. 
    def test_create_fretboard_style_with_invalid_fretboard_range_input(self):
        with pytest.raises(ValueError):
            style = FretboardTheme(fretboardRange=FretboardRange(0, 12))

    # Tests that a FretboardStyle object cannot be created with an invalid fretboard design input. 
    def test_create_fretboard_style_with_invalid_fretboard_design_input(self):
        with pytest.raises(TypeError):
            style = FretboardTheme(fretboardDesign="invalid")

    # Tests that a FretboardStyle object has an orientation property.  
    def test_fretboard_style_has_orientation_property(self):
        style = FretboardTheme()
        assert hasattr(style, "orientation")

    # Tests that a FretboardStyle object can set and get properties for each of its components.  
    def test_fretboard_style_can_set_and_get_properties(self):
        style = FretboardTheme(orientation=FretboardOrientation(orientation="v"), tuning=Tuning(numOfStrings=7), fretboardRange=FretboardRange(1, 12), fretboardDesign=FretboardDesign(distanceBetweenFrets=6))
        assert style.orientation.orientation == "v"
        assert style.tuning.numOfStrings == 7
        assert style.fretboardRange.fretFrom == 1
        assert style.fretboardDesign.distanceBetweenFrets == 6

    # Tests the interaction between the FretboardStyle object and its components.  
    def test_fretboard_style_components_interaction(self):
        style = FretboardTheme(orientation="h", tuning=Tuning(numOfStrings=4), fretboardRange=FretboardRange(1, 5), fretboardDesign=FretboardDesign(distanceBetweenFrets=3))
        assert style.orientation.orientation == "h"
        assert style.tuning.numOfStrings == 4
        assert style.fretboardRange.fretFrom == 1
        assert style.fretboardRange.numOfFrets == 4
        assert style.fretboardDesign.distanceBetweenFrets == 3
        assert style.fretboardDesign.showTuning == True

    # Tests the behavior of the Tuning class.   
    def test_tuning_class_behavior(self):
        # Test default values
        tuning = Tuning()
        assert tuning.letterTuning == STANDARD_TUNING
        assert tuning.midiTuning == STANDARD_TUNING_MIDI
        assert tuning.numOfStrings == 6

        # Test custom values
        tuning = Tuning(letterTuning=['D', 'G', 'C', 'F', 'A', 'D'], midiTuning=[38, 43, 48, 53, 57, 62], numOfStrings=6)
        assert tuning.letterTuning == ['D', 'G', 'C', 'F', 'A', 'D']
        assert tuning.midiTuning == [38, 43, 48, 53, 57, 62]
        assert tuning.numOfStrings == 6

    # Tests the behavior of the FretboardRange class.   
    def test_fretboard_range_class_behavior(self):
        # Test valid input
        fretboardRange = FretboardRange(1, 12)
        assert fretboardRange.fretFrom == 1
        assert fretboardRange.fretTo == 12
        assert fretboardRange.numOfFrets == 11

        # Test invalid input
        with pytest.raises(ValueError):
            fretboardRange = FretboardRange(-1, 12)
        with pytest.raises(ValueError):
            fretboardRange = FretboardRange(12, 1)

    # Tests the behavior of the FretboardDesign class.    
    def test_fretboard_design_class_behavior(self):
        design = FretboardDesign()
        assert design.showTuning == True
        assert design.showFretboardNumber == True
        assert design.distanceBetweenFrets == 5
        assert design.distanceBetweenStrings == 2
        assert design.fretColor == "black"
        assert design.stringsColor == "black"
        assert design.fretOpacity == 0.3
        assert design.stringsOpacity == 1
        assert design.fretboardMarkerColor == "#DCDCDC"

    # Tests that the fretboardDesign property getter returns the correct value.    
    def test_fretboard_design_property_getter(self):
        style = FretboardTheme(fretboardDesign=FretboardDesign(showTuning=False))
        assert style.fretboardDesign.showTuning == False